/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.console.de;


/**
 * Die Schnittstelle IKonsole definiert Methoden zur Ein- und Ausgabe von Daten
 * der Typen String (=Zeichenkette), int (=Ganzzahl) und float (=Fließkommazahl)
 * im Rahmen einfacher Konsolenanwendungen.
 * 
 * @author Frank Gollnik 
 * @version 1.2.1
 */

public interface IKonsole
{
    /**
     * Einlesen einer Textzeile.
     * Der Anwender wird zu einer neuen Eingabe aufgefordert, falls die Textzeile leer ist.
     * 
     * @return        die eingelesene Textzeile
     */
    public String leseText();

    /**
     * Einlesen einer Textzeile.
     * Der Anwender wird zu einer neuen Eingabe aufgefordert, falls die Textzeile leer ist.
     * 
     * @param   eingabeaufforderung   die Eingabeaufforderung, die dem Anwender vor der Eingabe angezeigt wird
     * @return                        die eingelesene Textzeile
     */
    public String leseText(String eingabeaufforderung);

    /**
     * Einlesen einer Ganzzahl.
     * Der Anwender wird zu einer neuen Eingabe aufgefordert, falls die Eingabe nicht das richtige Format besitzt.
     * 
     * @return        die eingelesene Ganzzahl
     */
    public int leseGanzzahl();

    /**
     * Einlesen einer Ganzzahl.
     * Der Anwender wird zu einer neuen Eingabe aufgefordert, falls die Eingabe nicht das richtige Format besitzt.
     * 
     * @param   eingabeaufforderung   die Eingabeaufforderung, die dem Anwender vor der Eingabe angezeigt wird
     * @return                        die eingelesene Ganzzahl
     */
    public int leseGanzzahl(String eingabeaufforderung);

    /**
     * Einlesen einer Fließkommazahl.
     * Der Anwender wird zu einer neuen Eingabe aufgefordert, falls die Eingabe nicht das richtige Format besitzt.
     * 
     * @return        die eingelesene Fließkommazahl
     */
    public float leseZahl();

    /**
     * Einlesen einer Fließkommazahl.
     * Der Anwender wird zu einer neuen Eingabe aufgefordert, falls die Eingabe nicht das richtige Format besitzt.
     * 
     * @param   eingabeaufforderung   die Eingabeaufforderung, die dem Anwender vor der Eingabe angezeigt wird
     * @return                        die eingelesene Fließkommazahl
     */
    public float leseZahl(String eingabeaufforderung);

    /**
     * Ausgabe eines Textes ohne Zeilenumbruch.
     * 
     * @param  text    der auszugebende Text
     */
    public void schreibe(String text);

    /**
     * Ausgabe einer Ganzzahl ohne Zeilenumbruch.
     * 
     * @param  ganzzahl    die auszugebende Ganzzahl
     */
    public void schreibe(int ganzzahl);

    /**
     * Ausgabe eine Fließkommazahl ohne Zeilenumbruch.
     * 
     * @param  fliesskommazahl    die auszugebende Fließkommazahl
     */
    public void schreibe(float fliesskommazahl);

    /**
     * Ausgabe eines Textes mit Zeilenumbruch.
     * 
     * @param  text    der auszugebende Text
     */
    public void schreibeZeile(String text);

    /**
     * Ausgabe einer Ganzzahl mit Zeilenumbruch.
     * 
     * @param  ganzzahl    die auszugebende Ganzzahl
     */
    public void schreibeZeile(int ganzzahl);

    /**
     * Ausgabe eine Fließkommazahl mit Zeilenumbruch.
     * 
     * @param  fliesskommazahl    die auszugebende Fließkommazahl
     */
    public void schreibeZeile(float fliesskommazahl);
}
