/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


import org.jabotics.robot.de.*;


/**
 * Beispielprogramm für die Ausgabe auf dem Monitor des Computerbausteins.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class B02_GiveMe5 extends XRoboter
{
    ////////////////////////////////////////////////////////////////////////////////////////
    // Hier werden die Objekte zur Robotersteuerung bereitgestellt.
    // Die Erzeugung der Objekte übernimmt für uns eine spezielle Klasse XObjektfabrik:
    ////////////////////////////////////////////////////////////////////////////////////////
    static final IMotor linkerMotor = objektfabrik.erzeugeMotor(EMotoranschluss.C);
    static final IMotor rechterMotor = objektfabrik.erzeugeMotor(EMotoranschluss.B);
    static final IPilot pilot = objektfabrik.erzeugePiloten(EMotoranschluss.C, EMotoranschluss.B);
    static final ITastsensor eingabetaste = objektfabrik.erzeugeTastsensor(ETaster.EINGABE);
    static final ITastsensor endetaste = objektfabrik.erzeugeTastsensor(ETaster.ENDE);
    static final ITastsensor linkstaste = objektfabrik.erzeugeTastsensor(ETaster.LINKS);
    static final ITastsensor rechtstaste = objektfabrik.erzeugeTastsensor(ETaster.RECHTS);
    static final ITastsensor tastsensor = objektfabrik.erzeugeTastsensor(ESensoranschluss.S1);
    static final ISensor schallsensor = objektfabrik.erzeugeSchallsensor(ESensoranschluss.S2);
    static final ILichtsensor lichtsensor = objektfabrik.erzeugeLichtsensor(ESensoranschluss.S3);
    static final ISensor ultraschallsensor = objektfabrik.erzeugeUltraschallsensor(ESensoranschluss.S4);
    static final IUhr uhr = objektfabrik.erzeugeUhr();
    static final IMonitor monitor = objektfabrik.erzeugeMonitor();
    static final ILautsprecher lautsprecher = objektfabrik.erzeugeLautsprecher();
 
     /**
     * Die Methode "main" ist der Startpunkt für das Roboterprogramm.
     * Diese Methode enthält die eigentliche Robotersteuerung, die du selbst programmieren sollst.
     * Ersetze das kleine Beispielprogramm durch deine eigenen Steuerungsbefehle.
     */
    public static void main(String[] args)
    {
        ////////////////////////////////////////////////////////////////////////////////////////
        // Beispiel-Robotersteuerung:
        ////////////////////////////////////////////////////////////////////////////////////////

        // Der Anwender soll die Eingabetaste 5 mal drücken
        monitor.loescheAlles();  // Löschen des Bildschirms kann hier nicht schaden.
        monitor.schreibe("Die Eingabetaste", 0, 0);    // Ausgabe der 1. Textzeile. Nummerierung der Zeilen beginnt bei 0
        monitor.schreibe("    muss noch", 0, 1);       // Ausgabe der 2. Textzeile
        monitor.schreibe("gedrueckt werden", 0, 3);    // Ausgabe der 4. Textzeile. Keine Umlaute erlaubt!
        monitor.schreibe("!!!", 7, 4);                 // Ausgabe ab der 8. Zeichenposition in der 5. Textzeile
        int n = 5;      // Der Zähler n speichert, wie oft die Eingabetaste noch gedrückt werden muss.
        do
        {   // Beginn des Schleifenblocks            // 
            monitor.schreibe(" " + n + " mal ", 5, 2, true);
                // Wiederholte Ausgabe der verbleibenden Anzahl n ab der 6. Position in der 3. Textzeile.
                // Die Ganzzahl n wird hier automatisch in einen Text umgewandelt (z.B. aus 5 wird "5").
                // Invertierte Darstellung zur Hervorhebung (siehe Dokumentation).
            monitor.aktualisiereAnzeige();   // sorgt für eine schnellere Aktualisierung des Bildschirms
            eingabetaste.warteBisEreignisLosgelassen();     // Programm wartet, bis die Eingabetaste
                                                            // gedrückt und wieder losgelassen wird.
            n = n - 1;  // Eingabetaste losgelassen!  =>  Zähler wird heruntergezählt.
        }   // Ende des Schleifenblocks
        while (n > 0);   // Die Schleife wird 5 mal durchlaufen, denn n wird bei jedem Durchlauf um 1 verringert (s.o.).
        
        // Fertig: Der Anwender hat 5 mal die Eingabetaste gedrückt und losgelassen.
        monitor.loescheAlles();
        monitor.schreibe("DANKE!", 5, 3);   // Dank ab 6. Position in 4. Zeile.
        uhr.warteZeit(3000); // Der Dank wird 3 Sekunden lang angezeigt, ehe das Programm endet
                             // und die Anzeige damit automatisch gelöscht bzw. überschrieben wird.
    }
}

