/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


import org.jabotics.robot.de.*;


/**
 * Beispielprogramm für den Einsatz des Lichtsenors und des Piloten.
 * Aufbau: Der Lichtsensor muss vorne am Roboter angebracht werden
 * und nach unten gerichtet sein.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class B04_FollowMe extends XRoboter
{
    ////////////////////////////////////////////////////////////////////////////////////////
    // Hier werden die Objekte zur Robotersteuerung bereitgestellt.
    // Die Erzeugung der Objekte übernimmt für uns eine spezielle Klasse XObjektfabrik:
    ////////////////////////////////////////////////////////////////////////////////////////
    static final IMotor linkerMotor = objektfabrik.erzeugeMotor(EMotoranschluss.C);
    static final IMotor rechterMotor = objektfabrik.erzeugeMotor(EMotoranschluss.B);
    static final IPilot pilot = objektfabrik.erzeugePiloten(EMotoranschluss.C, EMotoranschluss.B);
    static final ITastsensor eingabetaste = objektfabrik.erzeugeTastsensor(ETaster.EINGABE);
    static final ITastsensor endetaste = objektfabrik.erzeugeTastsensor(ETaster.ENDE);
    static final ITastsensor linkstaste = objektfabrik.erzeugeTastsensor(ETaster.LINKS);
    static final ITastsensor rechtstaste = objektfabrik.erzeugeTastsensor(ETaster.RECHTS);
    static final ITastsensor tastsensor = objektfabrik.erzeugeTastsensor(ESensoranschluss.S1);
    static final ISensor schallsensor = objektfabrik.erzeugeSchallsensor(ESensoranschluss.S2);
    static final ILichtsensor lichtsensor = objektfabrik.erzeugeLichtsensor(ESensoranschluss.S3);
    static final ISensor ultraschallsensor = objektfabrik.erzeugeUltraschallsensor(ESensoranschluss.S4);
    static final IUhr uhr = objektfabrik.erzeugeUhr();
    static final IMonitor monitor = objektfabrik.erzeugeMonitor();
    static final ILautsprecher lautsprecher = objektfabrik.erzeugeLautsprecher();
 
     /**
     * Die Methode "main" ist der Startpunkt für das Roboterprogramm.
     * Diese Methode enthält die eigentliche Robotersteuerung, die du selbst programmieren sollst.
     * Ersetze das kleine Beispielprogramm durch deine eigenen Steuerungsbefehle.
     */
    public static void main(String[] args)
    {
        ////////////////////////////////////////////////////////////////////////////////////////
        // Beispiel-Robotersteuerung:
        ////////////////////////////////////////////////////////////////////////////////////////

        uhr.warteZeit(2000);  // Zeit, um die Hand nach dem Programmstart aus dem Weg zu nehmen
        pilot.setzeZaehlerstaendeZurueck();  // Zurücksetzen des Tachostands
        pilot.setzeSollgeschwindigkeit(10);  // mäßig große Sollgeschwindigkeit von 10cm/s
        lichtsensor.setzeTraegheit(100);  // Der Sensor soll nicht zu schnell auf Helligkeitsschwankungen (z.B. durch Schmutz) reagieren.
                                          // Die angegebene Zeit von 100 ms ist der Mittelungszeitraum.
        lichtsensor.schalteLichtEin();  // Die Lichtquelle des Sensors soll den Boden beleuchten
        int helligkeitsgrenze = 50;  // Helligkeitsgrenze für das Erkennen der Kante der Linie.
                                     // Der Wert kann bei Bedarf an die Gegebenheiten angepasst werden.
                                     // Wert 0 entspricht sehr dunkel, 100 entspricht sehr hell.
        int radius = 10;  // Kurvenradius in cm für die folgenden Kurvenfahrten

        do   // Beginn einer Schleife
        {
            pilot.fahreRadius(radius,VORWAERTS);   // Kurvenfahrt nach links vorwärts mit dem angegebenen Kurvenradius.
            lichtsensor.warteBisGrenzwertUeberschritten(helligkeitsgrenze);  // Fahrt nach links fortsetzen, solange Boden dunkel
            pilot.fahreRadius(-radius,VORWAERTS);   // Kurvenfahrt nach rechts vorwärts mit dem angegebenen Kurvenradius.
            lichtsensor.warteBisGrenzwertUnterschritten(helligkeitsgrenze);  // Fahrt nach rechts fortsetzen, solange Boden hell
        }
        while (pilot.holeTachostand() < 1000);   // Der Tachostand (in cm) wird überprüft.
        pilot.halteAn(ABWARTEN);
        lichtsensor.schalteLichtAus();

        //Die gefahrenen Strecke soll noch angezeigt werden:
        float strecke = pilot.holeTachostand(); // Hier wird der Tachostand ausgelesen und in der neuen Fließkomma-Variablen strecke gespeichert.
        monitor.loescheAlles();
        monitor.schreibe("Strecke in cm:", 0, 0);
        monitor.schreibe(strecke, 0, 1);
        monitor.schreibe("Bitte", 4, 3);
        monitor.schreibe("Endetaste", 2, 4);
        monitor.schreibe("druecken!", 3, 5);

        lautsprecher.spieleBeepfolgeAufwaerts();

        endetaste.warteBisEreignisLosgelassen();
    }
}
