/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 * 
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.pc.swing;

import java.awt.Rectangle;

/**
 * XClipArea is a helper class for calculating total clipping areas.
 * On windows systems, sucsessive invokations of the repaint method
 * does not seem to calculate the correct clipping region.
 * => Calculate the total clipping rectangle and make a single call
 *    to repaint when updating dynamic graphics.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class XClipArea
{
    private Rectangle mRect = null;

    public Rectangle get()
    {
        return mRect;
    }

    public boolean isEmpty()
    {
        return (null == mRect);
    }

    public void clear()
    {
        mRect = null;
    }

    public void set(int x, int y, int width, int height)
    {
        mRect = new Rectangle(x,y,width,height);
    }

    public void set(Rectangle rect)
    {
        mRect = rect;
    }

    public void add(int x, int y, int width, int height)
    {
        add(new Rectangle(x,y,width,height));
    }

    public void add(Rectangle rect)
    {
        mRect = (null == mRect) ? rect : mRect.union(rect);
    }
}
