/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.de;

import org.jabotics.robot.defs.*;
import org.jabotics.robot.adapter.XAdapter;

/**
 * Objekte vom Typ XMonitor repräsentieren den Monitor des NXT-Bausteins
 * und können über die Schnittstelle {@link IMonitor} angesteuert werden.
 * Der Konstruktor der Klasse ist "nicht öffentlich". Man kann deshalb
 * als Anwender selbst keine Objekte dieses Typs erzeugen.
 * Man verschafft sich Zugang zu einem neuen Monitor-Objekt, indem man die Methode
 * {@link IObjektfabrik#erzeugeMonitor()} aufruft.
 * Man erhält Zugriff auf die einzige Objektfabrik-Instanz, indem man
 * die statische Methode {@link XObjektfabrik#holeObjektfabrik()} aufruft.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
class XMonitor implements IMonitor
{
    private final IDisplayAdapter mDisplay = XAdapter.getFactory().getDisplayAdapter();

    protected XMonitor()
    {
    }

    public void aktualisiereAnzeige()
    {
        mDisplay.refresh();
    }
    
    public void loescheAlles()
    {
        mDisplay.clear();
    }
    
    public void loescheZeile(int y_position)
    {
        mDisplay.clear(y_position);
    }
    
    public void loescheZeichen(int x_position, int y_position, int anzahl)
    {
        mDisplay.clear(x_position, y_position, anzahl);
    }
    
    public void schreibe(char zeichen, int x_position, int y_position)
    {
        mDisplay.draw(zeichen, x_position, y_position);
    }
    
    public void schreibe(int ganzzahl, int x_position, int y_position)
    {
        mDisplay.draw(ganzzahl, x_position, y_position);
    }
    
    public void schreibe(int ganzzahl, int x_position, int y_position, int anzahl_stellen)
    {
        mDisplay.draw(ganzzahl, x_position, y_position, anzahl_stellen);
    }
    
    public void schreibe(float fliesskommazahl, int x_position, int y_position)
    {
        mDisplay.draw(fliesskommazahl, x_position, y_position);
    }
    
    public void schreibe(String text, int x_position, int y_position)
    {
        mDisplay.draw(text, x_position, y_position);
    }
    
    public void schreibe(String text, int x_position, int y_position, boolean hervorgehoben)
    {
        mDisplay.draw(text, x_position, y_position, hervorgehoben);
    }
}
