/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.defs;


/**
 * The simulation mode needs to be defined prior to the first usage of the
 * AdapterFactory class. For changing the simulation mode from its default,
 * use the static method org.jabotics.sim.SIMAdapter.setSimulationMode(...).
 * This will only be necessary for the DEMO and the HIDDEN mode.
 * Since a robot class will typically derive from org.jabotics.en.XRobot
 * or org.jabotics.de.XRoboter, the AdapterFactory instance will automatically
 * be accessed during class initialization. Therefore for using the DEMO or
 * HIDDEN mode, an encapsulation class like a unit testing class will need
 * to set the simulation mode prior to accessing such robot class(es). 
 * 
 * NORMAL: Simulation and robot session(s) run in different processes.
 *         This mode will automatically be used when starting the simulation
 *         in a new process and passing the robot class name and possibly
 *         a class path to XSimulation.main(...).
 * TEST:   This mode will be used by default when starting the simulation by calling
 *         the main method of the robot class.
 *         Simulation and robot run in the same process.
 *         The UI is visible and interactive.
 *         The robot session will only run once. It will not be possible
 *         to reset and restart the robot session from the UI.
 *         This mode will typically be used for debugging purposes.
 * DEMO:   The same as for TEST, but the UI will not be interactive.
 *         The robot session has to be controlled programmatically via the interface
 *         ISessionAdapter. This mode can be used for demo runs of robot simulations.
 * HIDDEN  The same as for DEMO, but the UI will not be displayed.
 *         This mode has been designed for automated unit testing.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public enum ESimulationMode
{
   NORMAL, TEST, DEMO, HIDDEN
}
