/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.en;


/**
 * <span class="beginner">&bull;</span> The interface ISpeaker provides methods for playing signals and tones
 * on the speaker of the robot brick.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */

public interface ISpeaker
{
    /**
     * <span class="beginner">&bull;</span> Returns the currently stored value for the default volume.
     * The possible range is from 0 (silent) to 100 (loud).
     * 
     * @return  the currently stored value for the default volume
     */
    public int getVolume();

    /**
     * <span class="beginner">&bull;</span> Sets the default volume to the given value.
     * The possible range is from 0 (silent) to 100 (loud).
     * 
     * @param   volume     the new value for the default volume
     */
    public void setVolume(int volume);

    /**
     * <span class="beginner">&bull;</span> Plays a beep.
     * Program execution will be continued only after the end of the beep.
     */
    public void playBeep();

    /**
     * <span class="beginner">&bull;</span> Plays two beeps.
     * Program execution will be continued only after the end of the beeps.
     */
    public void playTwoBeeps();

    /**
     * <span class="beginner">&bull;</span> Plays a beep sequence.
     * Program execution will be continued only after the end of the beep sequence.
     */
    public void playBeepSequence();

    /**
     * <span class="beginner">&bull;</span> Plays an upward beep sequence.
     * Program execution will be continued only after the end of the beep sequence.
     */
    public void playBeepSequenceUp();

    /**
     * <span class="beginner">&bull;</span> Plays a buzz.
     * Program execution will be continued only after the end of the buzz.
     */
    public void playBuzz();

    /**
     * <span class="beginner">&bull;</span> Plays a tone.
     * Program execution will be continued only after the end of the tone.
     * 
     * @param   frequency    frequency of the tone
     * @param   period       duration of the tone (in ms)
     */
    public void playTone(int frequency, int period);

    /**
     * <span class="beginner">&bull;</span> Plays a tone.
     * The volume (range from 0 to 100) is relative to the stored value for the default volume.
     * Program execution will be continued only after the end of the tone.
     * 
     * @param   frequency    frequency of the tone
     * @param   period       duration of the tone (in ms)
     * @param   volume       relative volume to play the tone
     */
    public void playTone(int frequency, int period, int volume);

    /**
     * <span class="advanced">&bull;</span> Plays a tone with a given sound characteristics.
     * Program execution will be continued only after the end of the note.
     * For the NXT and EV3 robot, the sound modulation is deactivated in the current version.
     * A plain tone will be played instead.
     * 
     * @param   sound        the sound characteristics of the tone
     * @param   frequency    frequency of the tone
     * @param   period       duration of the tone (in ms)
     * @see     ESound
     */
    public void playNote(ESound sound, int frequency, int period);

    /**
     * <span class="advanced">&bull;</span> Plays a pause.
     * Program execution will be continued only after the end of the pause.
     * Alternatively, use {@link ITimer#wait(int)}.
     * 
     * @param   period       duration of the pause (in ms)
     */
    public void pause(int period);
}
