/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.en;

import org.jabotics.robot.core.CMotor;

/**
 * Objects of type XMotor represent a motor of the robot.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
class XMotor implements IMotor
{
    // Instanzvariablen
    private final CMotor mMotor;
    private final CMotor.XTransaction mMotorTransaction;

    XMotor(org.jabotics.robot.en.EMotorPort port)
    {
        mMotor = CMotor.getMotor(org.jabotics.robot.defs.EMotorPort.values()[port.ordinal()]);
        mMotorTransaction = new CMotor.XTransaction(mMotor);
    }

    @Override
    public void setCorrectionFactor(float correctionFactor)
    {
        mMotor.setTachoCountCorrectionFactor(correctionFactor);
    }

    @Override
    public float getPosition()
    {
        return Math.round(mMotor.getPosition());
    }

    @Override
    public void resetPosition()
    {
        mMotorTransaction.start();
        mMotor.resetPosition();
        mMotorTransaction.commit(false);
    }
    
    @Override
    public float getSpeed()
    {
        return mMotor.getSpeed();
    }
    
    @Override
    public float getTargetSpeed()
    {
        return mMotor.getTargetSpeed();
    }
    
    @Override
    public void setTargetSpeed(float speed)
    {
        mMotorTransaction.start();
        mMotor.setTargetSpeed(mMotorTransaction,speed);
        mMotorTransaction.commit(false);
    }
    
    @Override
    public float getTargetAcceleration()
    {
        return mMotor.getTargetAcceleration();
    }
    
    @Override
    public void setTargetAcceleration(float acceleration)
    {
        mMotorTransaction.start();
        mMotor.setTargetAcceleration(mMotorTransaction,acceleration);
        mMotorTransaction.commit(false);
    }

    @Override
    public float getMaxSpeed()
    {
        return mMotor.getMaxSpeed();
    }
    
    @Override
    public void setTargetSpeedToMax()
    {
        mMotorTransaction.start();
        mMotor.setTargetSpeedToMax(mMotorTransaction);
        mMotorTransaction.commit(false);
    }
    
    @Override
    public float getMaxAcceleration()
    {
        return mMotor.getMaxAcceleration();
    }

    @Override
    public void setTargetAccelerationToMax()
    {
        mMotorTransaction.start();
        mMotor.setTargetAccelerationToMax(mMotorTransaction);
        mMotorTransaction.commit(false);
    }

    @Override
    public void rotate(EDirection direction)
    {
        mMotorTransaction.start();
        mMotor.startRotationUnlimited(mMotorTransaction, EDirection.FORWARD == direction);
        mMotorTransaction.commit(false);
    }

    @Override
    public void rotateTime(float period, EDirection direction, EWaitingMode waitingMode)
    {
        mMotorTransaction.start();
        mMotor.startRotationTimeLimited(mMotorTransaction, period, EDirection.FORWARD == direction);
        mMotorTransaction.commit(EWaitingMode.WAIT == waitingMode);
    }

    @Override
    public void rotateByAngle(float angle, EWaitingMode waitingMode)
    {
        mMotorTransaction.start();
        mMotor.startRotationPosLimited(mMotorTransaction, angle, true);
        mMotorTransaction.commit(EWaitingMode.WAIT == waitingMode);
    }

    @SuppressWarnings("deprecation")
    @Override
    public void rotateToPostion(float position, EWaitingMode waitingMode)
    {
        rotateToPosition(position, waitingMode);
    }

    @Override
    public void rotateToPosition(float position, EWaitingMode waitingMode)
    {
        mMotorTransaction.start();
        mMotor.startRotationPosLimited(mMotorTransaction, position, false);
        mMotorTransaction.commit(EWaitingMode.WAIT == waitingMode);
    }

    @Override
    public void waitUntilRotationCompleted()
    {
        mMotorTransaction.waitUntilMotionCompleted();
    }
    
    @Override
    public void stopRotation(EWaitingMode waitingMode)
    {
        if (mMotor.isMoving())
        {
            mMotorTransaction.start();
            mMotor.stopRotation(mMotorTransaction);
            mMotorTransaction.commit(EWaitingMode.WAIT == waitingMode);
        }
    }
    
    @Override
    public void releaseControl()
    {
        mMotorTransaction.start();
        mMotor.setToFloatMode(mMotorTransaction);
        mMotorTransaction.commit(false);
    }
    
    @Override
    public boolean isMoving()
    {
        return mMotor.isMoving();
    }
    
    @Override
    public boolean isStalled()
    {
        return mMotor.isStalled();
    }
}
