/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.defs.*;
import java.awt.Color;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Path2D;

/**
 * Beschreiben Sie hier die Klasse XButton.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class XButton extends BSensor
{
    // Instanzvariablen
    private final Object mValueMonitor = new Object();
    private long mPressedUntil = 0;
    private float mValue = 0f;
    private EButtonType mType;

    /**
     * Konstruktor für Objekte der Klasse XButton
     */
    XButton(XRobot robot, EButtonType type)
    {
    	super(robot);

    	mType = type;
        //mPortID = mType.ordinal() + D.SENSOR_PORT_NUMBER;

        switch (type)
        {
            case ENTER:
            {
                XGraphicsShape enter = new XGraphicsShape(new Rectangle2D.Float(-1.5f,-1f,2f,2f)); // in robot coordinates
                enter.setPaint(new Color(255,190,90));
                enter.setDraw(false);
                enter.setFill(true);
                mGraphicsRep.add(enter);
                break;
            }
            case ESCAPE:
            {
                XGraphicsShape escape = new XGraphicsShape(new Rectangle2D.Float(1f,-1f,1.2f,2f)); // in robot coordinates
                escape.setPaint(new Color(153,153,153));
                escape.setDraw(false);
                escape.setFill(true);
                mGraphicsRep.add(escape);
                break;
            }
            case LEFT:
            {
                Path2D leftShape = new Path2D.Float();
                leftShape.setWindingRule(Path2D.WIND_EVEN_ODD);
                leftShape.moveTo(-1.5f,-1.5f); // in robot coordinates
                leftShape.lineTo(-0.5f,-3.0f);
                leftShape.lineTo(0.5f,-1.5f);
                leftShape.lineTo(-1.5f,-1.5f);
                XGraphicsShape left = new XGraphicsShape(leftShape);
                left.setPaint(new Color(221,221,221));
                left.setDraw(false);
                left.setFill(true);
                mGraphicsRep.add(left);
                break;
            }
            default:
            {
                Path2D rightShape = new Path2D.Float();
                rightShape.setWindingRule(Path2D.WIND_EVEN_ODD);
                rightShape.moveTo(-1.5f,1.5f); // in robot coordinates
                rightShape.lineTo(-0.5f,3.0f);
                rightShape.lineTo(0.5f,1.5f);
                rightShape.lineTo(-1.5f,1.5f);
                XGraphicsShape right = new XGraphicsShape(rightShape);
                right.setPaint(new Color(221,221,221));
                right.setDraw(false);
                right.setFill(true);
                mGraphicsRep.add(right);
                break;
            }
        }
    }

    @Override
    public void manageVisitor(BModelVisitor visitor)
    {
        visitor.visitButton(this);
    }

    @Override
    public ESensorType getType()
    {
        return ESensorType.TOUCH_SENSOR;
    }

    public EButtonType getButtonType()
    {
        return mType;
    }

    /*@Override
    public int getPortID()
    {
        return mType.ordinal() + D.SENSOR_PORT_NUMBER;
    }*/

    @Override
    public float getValue(XSensorRef ref)
    {
    	synchronized (mValueMonitor)
    	{
	    	if (0 == mPressedUntil) return mValue;
	    	if (mSession.currentTimeMillisRobot() >= mPressedUntil) return 1f;
	    	mPressedUntil = 0;
	    	return mValue;
    	}
    }

    void setPressedForPeriod(int period)
    {
    	if (0 > period) return;

    	synchronized (mValueMonitor)
    	{
	    	mPressedUntil = mSession.currentTimeMillisRobot() + period; 
	        mValue = 0f;
    	}
    }

    void setPressed()
    {
    	synchronized (mValueMonitor)
    	{
	    	mPressedUntil = 0;
    		mValue = 1f;
    	}
    }

    void setReleased()
    {
    	synchronized (mValueMonitor)
    	{
	    	mPressedUntil = 0;
    		mValue = 0f;
    	}
    }

    @Override
    public void initSession()
    {
        setReleased();
    }
}
