/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.defs.*;
import java.util.*;
import java.io.InputStream;
import java.io.IOException;
import java.awt.*;
import java.awt.geom.*;
import java.awt.font.*;
import javax.swing.text.*;

/**
 * Beschreiben Sie hier die Klasse XDisplay.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class XDisplay extends BModelElement implements IDisplayAdapter
{
    // class variables
    private static final int ROWS = 8;
    private static final int COLUMNS = 16;
    private static final int COLUMNS_PLUS_1 = COLUMNS + 1;
    private static final int FONT_SIZE = 12;
    //private static final int FONT_SIZE = 13;
    private static final Color FOREGROUND = new Color(23,24,21);
    private static final Color BACKGROUND = new Color(230,240,210);

    // instance variables:
    private DefaultStyledDocument mDocument;
    private FontRenderContext mDefaultFRC;
    private Font mFont;
    private Locale mLocaleDisplay;
    private SimpleAttributeSet mAttrSetNormal;
    private SimpleAttributeSet mAttrSetHighlighted;
    private final String mEmptyLine;

    XDisplay(XRobot robot)
    {
    	super(robot.getSession());

    	XGraphicsShape display = new XGraphicsShape(new Rectangle2D.Float(-6.5f,-3f,4f,6f)); // in robot coordinates
        display.setPaint(new Color(200,220,160));
        display.setDraw(false);
        display.setFill(true);
        mGraphicsRep.add(display);

        mDocument = new DefaultStyledDocument();
        mLocaleDisplay = new Locale("en","US");

        // create desired font if not yet available
        GraphicsEnvironment graphicsEnv = GraphicsEnvironment.getLocalGraphicsEnvironment();
        String[] fontNames = graphicsEnv.getAvailableFontFamilyNames(mLocaleDisplay);
        int i;
        for (i = 0;  i < fontNames.length;  ++i)
        {
            //if (fontNames[i].equals("Ubuntu Mono Robot")) break;
            if (fontNames[i].equals("Liberation Mono Robot")) break;
        }
        if (i == fontNames.length)
        {
            try
            {
                ClassLoader cl = this.getClass().getClassLoader();
                /*InputStream is = cl.getResourceAsStream("resources/fonts/UbuntuMonoRobot-Regular.ttf");
                Font font = Font.createFont(Font.TRUETYPE_FONT,is);
                graphicsEnv.registerFont(font);
                is = cl.getResourceAsStream("resources/fonts/UbuntuMonoRobot-Bold.ttf");
                font = Font.createFont(Font.TRUETYPE_FONT,is);
                graphicsEnv.registerFont(font);
                is = cl.getResourceAsStream("resources/fonts/UbuntuMonoRobot-Italic.ttf");
                font = Font.createFont(Font.TRUETYPE_FONT,is);
                graphicsEnv.registerFont(font);
                is = cl.getResourceAsStream("resources/fonts/UbuntuMonoRobot-BoldItalic.ttf");
                font = Font.createFont(Font.TRUETYPE_FONT,is);
                graphicsEnv.registerFont(font);*/
                InputStream is = cl.getResourceAsStream("resources/fonts/LiberationMonoRobot-Bold.ttf");
                Font font = Font.createFont(Font.TRUETYPE_FONT,is);
                graphicsEnv.registerFont(font);
                /*is = cl.getResourceAsStream("resources/fonts/LiberationMonoRobot-Regular.ttf");
                font = Font.createFont(Font.TRUETYPE_FONT,is);
                graphicsEnv.registerFont(font);
                is = cl.getResourceAsStream("resources/fonts/LiberationMonoRobot-Italic.ttf");
                font = Font.createFont(Font.TRUETYPE_FONT,is);
                graphicsEnv.registerFont(font);
                is = cl.getResourceAsStream("resources/fonts/LiberationMonoRobot-BoldItal.ttf");
                font = Font.createFont(Font.TRUETYPE_FONT,is);
                graphicsEnv.registerFont(font);*/
            }
            catch (FontFormatException ffe)
            {
                System.err.println(XMsg.ERROR);
                System.err.print(XMsg.CLASS);
                System.err.println(" XDisplay");
                System.err.print(XMsg.METHOD);
                System.err.println(" XDisplay()");
                System.err.print(XMsg.ERROR_TYPE);
                System.err.println(" FontFormatException");
                System.err.println(ffe.getMessage());
            }
            catch (IOException ioe)
            {
                System.err.println(XMsg.ERROR);
                System.err.print(XMsg.CLASS);
                System.err.println(" XDisplay");
                System.err.print(XMsg.METHOD);
                System.err.println(" XDisplay()");
                System.err.print(XMsg.ERROR_TYPE);
                System.err.println(" IOException");
                System.err.println(ioe.getMessage());
            }
        }
        
        // helper objects to estimate correct size of the display:
        //mDefaultFRC = new FontRenderContext(null,true,true);
        mDefaultFRC = new FontRenderContext(null,RenderingHints.VALUE_TEXT_ANTIALIAS_DEFAULT,RenderingHints.VALUE_FRACTIONALMETRICS_DEFAULT);
        mFont = new Font("Liberation Mono Robot", Font.BOLD, FONT_SIZE);
        //mFont = new Font("Ubuntu Mono Robot", Font.PLAIN, FONT_SIZE);

        // For available text attributes see documentation for class java.awt.font.TextAttribute
        // Helper class StyleConstants does not support all text attributes (like kerning)
        // Reason: JTextPane and JEditorPane do not support these attributes either!
        // => make font modification when necessary (here: Ubuntu Mono Robot)
        mAttrSetNormal = new SimpleAttributeSet();
        StyleConstants.setFontFamily(mAttrSetNormal, "Liberation Mono Robot");
        //StyleConstants.setFontFamily(mAttrSetNormal, "Ubuntu Mono Robot");
        StyleConstants.setFontSize(mAttrSetNormal, FONT_SIZE);
        StyleConstants.setBold(mAttrSetNormal, true);
        mAttrSetHighlighted = new SimpleAttributeSet(mAttrSetNormal);
        StyleConstants.setForeground(mAttrSetNormal, FOREGROUND);
        StyleConstants.setBackground(mAttrSetNormal, BACKGROUND);
        StyleConstants.setForeground(mAttrSetHighlighted, BACKGROUND);
        StyleConstants.setBackground(mAttrSetHighlighted, FOREGROUND);

        StringBuffer sb = new StringBuffer();
        for (int j = 0;  j < COLUMNS;  ++j) sb.append(' ');
        sb.append('\n');
        mEmptyLine = sb.toString();
        initDocument(); // mEmptyLine needs to be initialized first
    }

    private void initDocument()
    {
        clear();
    }

    //AbstractDocument getDocument()
    StyledDocument getDocument()
    {
        return mDocument;
    }

    Dimension getDisplaySize()
    {
        return getDisplaySize(mDefaultFRC);
    }

    Dimension getDisplaySize(FontRenderContext frc)
    {
        Rectangle2D rect = mFont.getStringBounds("MMMMMMMMMMMMMMMM",frc);
        return new Dimension((int)rect.getWidth()+2, (int)(ROWS*rect.getHeight())+4);
        //return new Dimension((int)rect.getWidth(), (int)(ROWS*(rect.getHeight()+1)+1));
    }

    Color getForeground()
    {
        return FOREGROUND;
    }

    Color getBackground()
    {
        return BACKGROUND;
    }

    @Override
    public void manageVisitor(BModelVisitor visitor)
    {
        visitor.visitDisplay(this);
    }

    @Override
    public void refresh()
    {
    }

    @Override
    public void clear()
    {
        try
        {
            mDocument.remove(0,mDocument.getLength());
            StringBuffer sb = new StringBuffer();
            for (int i = 0;  i < ROWS;  ++i) sb.append(mEmptyLine);
            mDocument.insertString(0,sb.toString(),mAttrSetNormal);
        }
        catch (BadLocationException e)
        {
            System.err.println(XMsg.ERROR);
            System.err.print(XMsg.CLASS);
            System.err.println(" XDisplay");
            System.err.print(XMsg.METHOD);
            System.err.println(" initDocument()");
            System.err.print(XMsg.ERROR_TYPE);
            System.err.println(" BadLocationException");
            System.err.println(e.getMessage());
        }
    }

    @Override
    public void clear(int y_position)
    {
        try
        {
            if ((y_position < 0) || (y_position >= ROWS)) return;
            int pos_start = y_position * COLUMNS_PLUS_1;
            mDocument.replace(pos_start,COLUMNS_PLUS_1,mEmptyLine,mAttrSetNormal);
        }
        catch (BadLocationException e)
        {
            System.err.println(XMsg.ERROR);
            System.err.print(XMsg.CLASS);
            System.err.println(" XDisplay");
            System.err.print(XMsg.METHOD);
            System.err.println(" clear(int y_position)");
            System.err.print(XMsg.ERROR_TYPE);
            System.err.println(" BadLocationException");
            System.err.println(e.getMessage());
        }
    }

    @Override
    public void clear(int x_position, int y_position, int count)
    {
        try
        {
            if ((x_position < 0) || (x_position >= COLUMNS) || (y_position < 0) || (y_position >= ROWS)) return;
            int pos_start = y_position * COLUMNS_PLUS_1 + x_position;
            int max_count = COLUMNS - x_position;
            count = (count > max_count) ? max_count : count;
            StringBuilder sb = new StringBuilder(COLUMNS_PLUS_1);
            for (int j = 0;  j < count;  ++j) sb.append(' ');
            mDocument.replace(pos_start,count,sb.toString(),mAttrSetNormal);
        }
        catch (BadLocationException e)
        {
            System.err.println(XMsg.ERROR);
            System.err.print(XMsg.CLASS);
            System.err.println(" XDisplay");
            System.err.print(XMsg.METHOD);
            System.err.println(" clear(int x_position, int y_position, int count)");
            System.err.print(XMsg.ERROR_TYPE);
            System.err.println(" BadLocationException");
            System.err.println(e.getMessage());
        }
    }

    @Override
    public void draw(char ch, int x_position, int y_position)
    {
        try
        {
            if ((x_position < 0) || (x_position >= COLUMNS) || (y_position < 0) || (y_position >= ROWS)) return;
            int pos_start = y_position * COLUMNS_PLUS_1 + x_position;
            StringBuilder sb = new StringBuilder(2);
            sb.append(ch);
            mDocument.replace(pos_start,1,sb.toString(),mAttrSetNormal);
        }
        catch (BadLocationException e)
        {
            System.err.println(XMsg.ERROR);
            System.err.print(XMsg.CLASS);
            System.err.println(" XDisplay");
            System.err.print(XMsg.METHOD);
            System.err.println(" clear(int x_position, int y_position, int count)");
            System.err.print(XMsg.ERROR_TYPE);
            System.err.println(" BadLocationException");
            System.err.println(e.getMessage());
        }
    }

    @Override
    public void draw(int n, int x_position, int y_position)
    {
        if ((x_position < 0) || (x_position >= COLUMNS) || (y_position < 0) || (y_position >= ROWS)) return;
        writeln(String.valueOf(n), x_position, y_position, false);
    }

    @Override
    public void draw(int n, int x_position, int y_position, int digits)
    {
        if ((x_position < 0) || (x_position >= COLUMNS) || (y_position < 0) || (y_position >= ROWS) || (digits <=0) ) return;
        String format_text = "%" + String.valueOf(digits) + "d";
        String text = String.format(mLocaleDisplay,format_text,n);
        writeln(text, x_position, y_position, false);
    }

    @Override
    public void draw(float fl, int x_position, int y_position)
    {
        if ((x_position < 0) || (x_position >= COLUMNS) || (y_position < 0) || (y_position >= ROWS)) return;
        String text = String.format(mLocaleDisplay,"%.7G",fl);
        writeln(text, x_position, y_position, false);
    }

    @Override
    public void draw(String text, int x_position, int y_position)
    {
        draw(text, x_position, y_position, false);
    }

    @Override
    public void draw(String text, int x_position, int y_position, boolean highlighted)
    {
        if ((x_position < 0) || (x_position >= COLUMNS) || (y_position < 0) || (y_position >= ROWS)) return;
        String[] lines = text.split("\n");
        for (int i = 0;  i < lines.length;  ++i)
        {
            writeln(lines[i],x_position,y_position, highlighted);
            x_position = 0; // next line starts at the beginning
            if (++y_position >= ROWS) break;
        }
    }

    private void writeln(String text, int x_position, int y_position, boolean highlighted)
    {
        // This method handles a single line in parameter text. There will be
        // - no check for valid locations,
        // - no (further) check for newline
        try
        {
            int count = text.length();
            if (0 == count) return;
            int pos_start = y_position * COLUMNS_PLUS_1 + x_position;
            int max_count = COLUMNS - x_position;
            count = (count > max_count) ? max_count : count;
            mDocument.replace(pos_start, count, text.substring(0,count).replace('\t',' '), highlighted ? mAttrSetHighlighted : mAttrSetNormal);
            //mDocument.replace(pos_start, count, text.substring(0,count).replace('\t',' '), mAttrSetNormal);
        }
        catch (BadLocationException e)
        {
            System.err.println(XMsg.ERROR);
            System.err.print(XMsg.CLASS);
            System.err.println(" XDisplay");
            System.err.print(XMsg.METHOD);
            System.err.println(" writeln(String text, int x_position, int y_position, boolean highlighted)");
            System.err.print(XMsg.ERROR_TYPE);
            System.err.println(" BadLocationException");
            System.err.println(e.getMessage());
        }
    }

    @Override
    public void initSession()
    {
        clear();
    }

    static public void test()
    {
        Locale locale = new Locale("en","US");
        float pi = (float)Math.PI;
        float fl = (float) (Math.PI * 123456789012345.0);
        String text1 = String.format(locale,"%.7f",pi);
        String text2 = String.format(locale,"%.7G",fl);
        String text3 = String.format(locale,"%.7G",pi);
        System.out.println(text1);
        System.out.println(text2);
        System.out.println(text3);
    }
}
