/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import java.util.Locale;
import java.util.ResourceBundle;

/**
 * Beschreiben Sie hier die Klasse XMsg
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class XMsg
{
    static final String VERSION_NUMBER =         "1.2.1";
    static final String COPYRIGHT =              "2012-2018";

    static boolean initialized = false;

    // initialize String with key (will be replaced in call to init-method)
    static String OK =                           "button.ok";
    static String CANCEL =                       "button.cancel";
    static String YES =                          "button.yes";
    static String NO =                           "button.no";
    static String APPLY =                        "button.apply";
    static String RESET =                        "button.reset";
    static String SET_DEFAULT =                  "button.default";
    static String IMPORT =                       "button.import";
    static String EXPORT =                       "button.export";
    static String START =                        "button.start";
    static String STOP =                         "button.stop";
    static String PAUSE =                        "button.pause";
    static String NEW =                          "button.new";
    static String WARNING =                      "warn.prompt";
    static String ERROR =                        "err.prompt";
    static String ERROR_TYPE =                   "err.type";
    static String INVALID_CALL_OF_MAIN =         "err.main";
    static String MAIN_FIRST_PARAM =             "err.param";
    static String MAIN_SECC_THIRD_PARAM =        "err.param2";
    static String CLASS =                        "err.class";
    static String INIT_CLASS_ERROR =             "err.class.init";
    static String METHOD =                       "err.method";
    static String ACCESS_METHOD_ERROR =          "err.method.access";
    static String PREFS_READ_ERROR =             "err.pref.read";
    static String PREFS_WRITE_ERROR =            "err.pref.write";
    static String PREFS_IMPORT_ERROR =           "err.pref.import";
    static String PREFS_EXPORT_ERROR =           "err.pref.export";
    static String WRONG_FILE_VERSION =           "err.file.version";
    static String INFO_MENU =                    "info.prompt";
    static String INFO =                         "info.title";
    static String VERSION =                      "info.version";
    static String AUTHOR =                       "info.author";
    static String PUBLISHED_UNDER =              "info.legal";
    static String CONTROL_MENU =                 "ctrl.prompt";
    static String SETTINGS_MENU =                "set.prompt";
    static String SIM_SETTINGS =                 "set.title";
    static String ROBOT_SIM =                    "sim.title";
    static String SIM_BREAKS =                   "sim.breaks.title";
    static String ENABLE_SIM_BREAKS =            "sim.breaks.prompt";
    static String ENABLE_SIM_BREAKS_INFO =       "sim.breaks.info";
    static String SUSPEND_ROBOT_THREAD =         "sim.thread.prompt";
    static String SUSPEND_ROBOT_THREAD_INFO =    "sim.thread.info";
    static String SENSORS =                      "sensor.prompt";
    static String PORT =                         "sensor.port";
    static String TYPE =                         "sensor.type.prompt";
    static String NO_SENSOR =                    "sensor.type.none";
    static String TOUCH_SENSOR =                 "sensor.type.touch";
    static String SOUND_SENSOR =                 "sensor.type.sound";
    static String LIGHT_SENSOR =                 "sensor.type.light";
    static String ULTRASONIC_SENSOR =            "sensor.type.ultrasonic";
    static String DIRECTION =                    "sensor.dir.prompt";
    static String NO_DIRECTION =                 "sensor.dir.none";
    static String FRONT =                        "sensor.dir.front";
    static String BACK =                         "sensor.dir.back";
    static String DOWNWARD =                     "sensor.dir.downward";
    static String FORWARD =                      "sensor.dir.forward";
    static String BACKWARD =                     "sensor.dir.backward";
    static String TO_LEFT =                      "sensor.dir.left";
    static String TO_RIGHT =                     "sensor.dir.right";
    static String SENSOR_NOT_DISPLAYED =         "sensor.invisible.info";
    static String ROBOT =                        "robot.prompt";
    static String START_POSITION =               "robot.start.pos";
    static String START_ORIENTATION =            "robot.start.orient";
    static String START_INFO =                   "robot.start.info";
    static String AUDIO_SYSTEM =                 "audio.system";
    static String AUDIO_IN =                     "audio.in";
    static String AUDIO_OUT =                    "audio.out";
    static String AUDIO_OFF =                    "audio.off";
    static String AUDIO_MIXER_DEFAULT =          "audio.mixer.default";
    static String AUDIO_MIXER_DEFAULT_INVALID =  "audio.mixer.default.invalid";
    static String AUDIO_MIXER_SELECT_INVALID =   "audio.mixer.select.invalid";

    static void init(Locale locale)
    {
        if (!initialized)
        {
            // do not allow the initialization of the simulation to crash due to a problem with the internationalization
            try
            {
                ResourceBundle messages = ResourceBundle.getBundle("resources/properties/jabotics-sim-messages", locale);

                // Replace key by translation
                OK =                           messages.getString(OK);
                CANCEL =                       messages.getString(CANCEL);
                YES =                          messages.getString(YES);
                NO =                           messages.getString(NO);
                APPLY =                        messages.getString(APPLY);
                RESET =                        messages.getString(RESET);
                SET_DEFAULT =                  messages.getString(SET_DEFAULT);
                IMPORT =                       messages.getString(IMPORT);
                EXPORT =                       messages.getString(EXPORT);
                START =                        messages.getString(START);
                STOP =                         messages.getString(STOP);
                PAUSE =                        messages.getString(PAUSE);
                NEW =                          messages.getString(NEW);
                WARNING =                      messages.getString(WARNING);
                ERROR =                        messages.getString(ERROR);
                ERROR_TYPE =                   messages.getString(ERROR_TYPE);
                INVALID_CALL_OF_MAIN =         messages.getString(INVALID_CALL_OF_MAIN);
                MAIN_FIRST_PARAM =             messages.getString(MAIN_FIRST_PARAM);
                MAIN_SECC_THIRD_PARAM =        messages.getString(MAIN_SECC_THIRD_PARAM);
                CLASS =                        messages.getString(CLASS);
                INIT_CLASS_ERROR =             messages.getString(INIT_CLASS_ERROR);
                METHOD =                       messages.getString(METHOD);
                ACCESS_METHOD_ERROR =          messages.getString(ACCESS_METHOD_ERROR);
                PREFS_READ_ERROR =             messages.getString(PREFS_READ_ERROR);
                PREFS_WRITE_ERROR =            messages.getString(PREFS_WRITE_ERROR);
                PREFS_IMPORT_ERROR =           messages.getString(PREFS_IMPORT_ERROR);
                PREFS_EXPORT_ERROR =           messages.getString(PREFS_EXPORT_ERROR);
                WRONG_FILE_VERSION =           messages.getString(WRONG_FILE_VERSION);
                INFO_MENU =                    messages.getString(INFO_MENU);
                INFO =                         messages.getString(INFO);
                VERSION =                      messages.getString(VERSION);
                AUTHOR =                       messages.getString(AUTHOR);
                PUBLISHED_UNDER =              messages.getString(PUBLISHED_UNDER);
                CONTROL_MENU =                 messages.getString(CONTROL_MENU);
                SETTINGS_MENU =                messages.getString(SETTINGS_MENU);
                SIM_SETTINGS =                 messages.getString(SIM_SETTINGS);
                ROBOT_SIM =                    messages.getString(ROBOT_SIM);
                SIM_BREAKS =                   messages.getString(SIM_BREAKS);
                ENABLE_SIM_BREAKS =            messages.getString(ENABLE_SIM_BREAKS);
                ENABLE_SIM_BREAKS_INFO =       messages.getString(ENABLE_SIM_BREAKS_INFO);
                SUSPEND_ROBOT_THREAD =         messages.getString(SUSPEND_ROBOT_THREAD);
                SUSPEND_ROBOT_THREAD_INFO =    messages.getString(SUSPEND_ROBOT_THREAD_INFO);
                SENSORS =                      messages.getString(SENSORS);
                PORT =                         messages.getString(PORT);
                TYPE =                         messages.getString(TYPE);
                NO_SENSOR =                    messages.getString(NO_SENSOR);
                TOUCH_SENSOR =                 messages.getString(TOUCH_SENSOR);
                SOUND_SENSOR =                 messages.getString(SOUND_SENSOR);
                LIGHT_SENSOR =                 messages.getString(LIGHT_SENSOR);
                ULTRASONIC_SENSOR =            messages.getString(ULTRASONIC_SENSOR);
                DIRECTION =                    messages.getString(DIRECTION);
                NO_DIRECTION =                 messages.getString(NO_DIRECTION);
                FRONT =                        messages.getString(FRONT);
                BACK =                         messages.getString(BACK);
                DOWNWARD =                     messages.getString(DOWNWARD);
                FORWARD =                      messages.getString(FORWARD);
                BACKWARD =                     messages.getString(BACKWARD);
                TO_LEFT =                      messages.getString(TO_LEFT);
                TO_RIGHT =                     messages.getString(TO_RIGHT);
                SENSOR_NOT_DISPLAYED =         messages.getString(SENSOR_NOT_DISPLAYED);
                ROBOT =                        messages.getString(ROBOT);
                START_POSITION =               messages.getString(START_POSITION);
                START_ORIENTATION =            messages.getString(START_ORIENTATION);
                START_INFO =                   messages.getString(START_INFO);
                AUDIO_SYSTEM =                 messages.getString(AUDIO_SYSTEM);
                AUDIO_IN =                     messages.getString(AUDIO_IN);
                AUDIO_OUT =                    messages.getString(AUDIO_OUT);
                AUDIO_OFF =                    messages.getString(AUDIO_OFF);
                AUDIO_MIXER_DEFAULT =          messages.getString(AUDIO_MIXER_DEFAULT);
                AUDIO_MIXER_DEFAULT_INVALID =  messages.getString(AUDIO_MIXER_DEFAULT_INVALID);
                AUDIO_MIXER_SELECT_INVALID =   messages.getString(AUDIO_MIXER_SELECT_INVALID);
            }
            catch (Exception e)
            {
                System.out.println ("Robot simulation: Internationialization of messages failed. Details:\n" + e.getMessage());
            }
            initialized = true;
        }
    }
}
