/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.pc.swing.XClipArea;
import java.awt.*;
import java.awt.geom.*;
import java.awt.image.BufferedImage;
import javax.swing.*;

/**
 * Beschreiben Sie hier die Klasse XSceneOverview.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class XSceneOverview extends JComponent
{
    // class variables
    final static float SCALE = 0.32f;    // pixel/cm
    //final static int SCENE_OFFSET_X = -25;  // cm
    //final static int SCENE_OFFSET_Y = -25;  // cm
    //final static int SCENE_WIDTH    = 850;  // cm
    //final static int SCENE_HEIGHT   = 650;  // cm
    final static int OFFSET_X = -8;           // pixel
    final static int OFFSET_Y = -8;           // pixel
    final static int WIDTH = 273;             // pixel
    final static int HEIGHT = 209;            // pixel
    final static int HALF_LINE_WIDTH = 2;
    final static int LINE_WIDTH = 2 * HALF_LINE_WIDTH;
    final static int DOUBLED_LINE_WIDTH = 2 * LINE_WIDTH;
    final static int RADIUS = 4;
    final static int DIAMETER = 2*RADIUS;
    final static int DOUBLED_DIAMETER = 2*DIAMETER;
    


    // instance variables
    XSceneView mSceneView;
    BufferedImage mSceneImg;
    BasicStroke mStroke;
    XRobot mRobot;
    int mRobotX;
    int mRobotY;
    Rectangle mRect;

    //Konstruktoren:
    public XSceneOverview(XSceneView sceneView)
    {
        super();

        Dimension dim = new Dimension(WIDTH,HEIGHT);
        setPreferredSize(dim);
        setMinimumSize(dim);
        setMaximumSize(dim);

        mSceneView = sceneView;
        mStroke = new BasicStroke(LINE_WIDTH,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND);

        paintSceneImage();

        mRobot = XSimulationModel.getRobotSession(0).getRobot();
        XRobot.XDynamicData dd = mRobot.getDynamicData();
        mRobotX = (int)(SCALE * dd.getRobotX() - OFFSET_X);
        mRobotY = (int)(SCALE * dd.getRobotY() - OFFSET_Y);
    }

    public void init()
    {
        //mSceneView.init();
        
        Rectangle2D.Float rect = mSceneView.getPhysicalViewRect();
        int x = (int)(SCALE * rect.x) - OFFSET_X;
        int y = (int)(SCALE * rect.y) - OFFSET_Y;
        int width  = (int)(SCALE * rect.width);
        int height = (int)(SCALE * rect.height);
        mRect = new Rectangle(x,y,width,height);
    }

    @Override
    public boolean isOpaque()
    {
        // This component is designed to be completely opaque.
        // By overriding the methode isOpaque() to deliver always true,
        // the parent container knows that it does not have to wipe out
        // the screen prior to repaint actions of this component.
        return true;
    }

    void updateGraphics()
    {
        XRobot.XDynamicData dd = mRobot.getDynamicData();
        int x = (int)(SCALE * dd.getRobotX()) - OFFSET_X;
        int y = (int)(SCALE * dd.getRobotY()) - OFFSET_Y;
        XClipArea clip = new XClipArea();
        if ((x != mRobotX) || (y != mRobotY))
        {
            clip.add(mRobotX-DIAMETER,mRobotY-DIAMETER,DOUBLED_DIAMETER,DOUBLED_DIAMETER);
            mRobotX = x;
            mRobotY = y;
            clip.add(mRobotX-DIAMETER,mRobotY-DIAMETER,DOUBLED_DIAMETER,DOUBLED_DIAMETER);
        }

        Rectangle2D.Float rect = mSceneView.getPhysicalViewRect();
        x = (int)(SCALE * rect.x) - OFFSET_X;
        y = (int)(SCALE * rect.y) - OFFSET_Y;
        int width = (int)(SCALE * rect.width);
        int height = (int)(SCALE * rect.height);
        if ((x != mRect.x) || (y != mRect.y) || (width != mRect.width) || (height != mRect.height))
        {
            clip.add(mRect.x-LINE_WIDTH, mRect.y-LINE_WIDTH, mRect.width+DOUBLED_LINE_WIDTH, mRect.height+DOUBLED_LINE_WIDTH);
            mRect = new Rectangle(x,y,width,height);
            clip.add(mRect.x-LINE_WIDTH, mRect.y-LINE_WIDTH, mRect.width+DOUBLED_LINE_WIDTH, mRect.height+DOUBLED_LINE_WIDTH);
        }

        if (!clip.isEmpty()) repaint(clip.get());
    }

    private void paintSceneImage()
    {
        mSceneImg = new BufferedImage(WIDTH,HEIGHT,BufferedImage.TYPE_INT_RGB);
        Graphics2D g2D = mSceneImg.createGraphics();
        AffineTransform aftInitial = g2D.getTransform();
        AffineTransform aft = new AffineTransform(SCALE, 0f, 0f, SCALE, (float)-OFFSET_X, (float)-OFFSET_Y);
        g2D.setTransform(aft);

        //XSimulationModel.getScene().paint(g2D);
        XScene scene = XSimulationModel.getScene();
        XFloor floor = scene.getFloor();
        g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
        floor.paint(g2D); // paints with opaque colors (=> alpha = 255)
        g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_OFF);
        floor.paintLuminosity(g2D,0.7f);
        g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
        scene.getWall().paint(g2D);

        g2D.setTransform(aftInitial);
    }

    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);

        Graphics2D g2D = (Graphics2D)g;

        Rectangle clip = g2D.getClipBounds();
        if (null == clip) clip = this.getBounds();

        if (!clip.isEmpty())
        {
            g2D.drawImage(mSceneImg.getSubimage(clip.x, clip.y, clip.width, clip.height), null, clip.x, clip.y);
            Color clr = g2D.getColor();
            g2D.setColor(Color.RED);
            Stroke sk = g2D.getStroke();
            g2D.setStroke(mStroke);
            g2D.drawRect(mRect.x-HALF_LINE_WIDTH,mRect.y-HALF_LINE_WIDTH,mRect.width+LINE_WIDTH,mRect.height+LINE_WIDTH);
            g2D.setStroke(sk);
            g2D.fillOval(mRobotX-RADIUS,mRobotY-RADIUS,DIAMETER,DIAMETER);
            g2D.setColor(clr);
        }
    }
}
