/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import javax.swing.*;
import javax.swing.event.*;

import org.jabotics.robot.defs.ESimulationMode;

import java.awt.*;
import java.awt.event.*;

/**
 * Beschreiben Sie hier die Klasse XSimulationControlPanel.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class XSimulationControlPanel extends JPanel implements ActionListener, XPreferences.IListener, ChangeListener
{
    private class UpdateGraphicsHandler implements Runnable
    {
        public void run()
        {
            /*int level = mSoundscape.getSoundLevel();
            if (level > 0) System.out.println(level);
            mBarInputLevel.setValue(level);*/
        	
            mBarInputLevel.setValue(mSoundscape.getSoundLevel());
        }
    }

    private class SessionHandler extends BSessionHandler
    {
        @Override
        public String getKey()
        {
        	return "XSimulationControlPanel.Session#" + mSession.getID();
        }

        @Override
        public void stopSession()
        {
                mButtonPause.setEnabled(false);
                mButtonStop.setText(XMsg.OK);
        }

        @Override
        public BSessionHandler completeSession()
        {
            if (XSimulation.getMode() == ESimulationMode.NORMAL)
            {
                mButtonPause.setText(XMsg.NEW);
                mButtonPause.setEnabled(true);
            }
            return this;
        }
    }

    private class ClapHandler extends MouseAdapter
    {
        @Override
        public void mousePressed(MouseEvent e)
        {
            if ( (e.getButton() == MouseEvent.BUTTON1) && (ESessionState.RUNNING == mSession.getState()) )
            {
                int h = mButtonClap.getHeight() - 1;
                int dy = h - e.getY();
                int level = (101 * dy) / h;
                if (level > 100)
                {
                    level = 100;
                }
                else if (level < 0)
                {
                    level = 0;
                }
                mSoundscape.addClap(level);
            }
        }
    }

    class SessionResetHandler extends SwingWorker<Boolean, Object>
    {
        // doInBackground() is performed in a worker thread.
        // Looks like the reset of the simulation needs to be performed outside the Event Dispatch Thread.
        // Otherwise there are problems with the update of the JTextArea representing the robot display.
        // During the reset the document in XDisplay is reinitialized. Possibly the corresponding change
        // notifications from the document cannot be handled in the right way by the JTextArea instance.
        @Override
        public Boolean doInBackground()
        {
            //XTimer.msDelay(100);
            mSession.init();//Bisher: XSimulation.resetRobot();
            return true;
        }

        // done() is performed in Event Dispatch Thread
        // Swing components need to be accessed in this thread.
        @Override
        protected void done()
        {
            try
            {
                mButtonStop.setEnabled(true);
                mButtonStart.setEnabled(true);
            }
            catch (Exception ignore)
            {
            }
        }
    }

    // instance variables
    private final XSession mSession = XSimulationModel.getRobotSession(0);
    private final UpdateGraphicsHandler mUpdateGraphicsHandler = new UpdateGraphicsHandler();
    private final XSoundscape mSoundscape = XSimulationModel.getSoundscape();
    private final boolean mIsInteractive;
    private boolean mChangingPreferences = false;
    private JButton mButtonClap;
    private JProgressBar mBarInputLevel;
    private JSlider mSliderInputLevel;
    private XBrickControl mBrickControl;
    private JButton mButtonStart;
    private JButton mButtonPause;
    private JButton mButtonStop;

    /**
     * Konstruktor für Objekte der Klasse XSimulationControlPanel
     */
    public XSimulationControlPanel(boolean interactive)
    {
        super(new GridBagLayout());

        mIsInteractive = interactive;

        // Create icons
        ClassLoader cl = this.getClass().getClassLoader();
        Icon iconCanClap = new ImageIcon(cl.getResource("resources/images/CanClap48x200.gif"));
        Icon iconClapped = new ImageIcon(cl.getResource("resources/images/Clapped48x200.gif"));
        Icon iconSound = new ImageIcon(cl.getResource("resources/images/Sound72x48.gif"));

        GridBagConstraints c;
        Insets noMargin = new Insets(0,0,0,0);
        //Insets smallMargin = new Insets(1,1,1,1);
        Insets stdMargin = new Insets(2,2,2,2);
        Dimension dimBar = new Dimension(30,200);
        Dimension dimSlider = new Dimension(30,214);

        JPanel paneSound = new JPanel(new GridBagLayout());
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1.0;
        c.weighty = 1.0;
        c.fill = GridBagConstraints.NONE;
        c.insets = stdMargin;
        this.add(paneSound,c);

        XPreferences.Read p = XPreferences.getReadInstance();
        mSliderInputLevel = new JSlider(SwingConstants.VERTICAL, 0, 100, p.getAudioInputLevel());
        mSliderInputLevel.setPreferredSize(dimSlider);
        mSliderInputLevel.setMinimumSize(dimSlider);
        mSliderInputLevel.setMaximumSize(dimSlider);
        if (mIsInteractive) mSliderInputLevel.addChangeListener(this);
        mSliderInputLevel.setEnabled(mIsInteractive);
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.fill = GridBagConstraints.NONE;
        c.insets = stdMargin;
        paneSound.add(mSliderInputLevel,c);

        mBarInputLevel = new JProgressBar(SwingConstants.VERTICAL, 0, 100);
        mBarInputLevel.setValue(p.getAudioInputLevel());
        mBarInputLevel.setPreferredSize(dimBar);
        mBarInputLevel.setMinimumSize(dimBar);
        mBarInputLevel.setMaximumSize(dimBar);
        c = new GridBagConstraints();
        c.gridx = 1;
        c.gridy = 0;
        c.fill = GridBagConstraints.NONE;
        c.insets = stdMargin;
        paneSound.add(mBarInputLevel,c);

        mButtonClap = new JButton(iconCanClap);
        mButtonClap.setPressedIcon(iconClapped);
        mButtonClap.setMargin(noMargin);
        if (mIsInteractive) mButtonClap.addMouseListener(new ClapHandler());
        mButtonClap.setEnabled(mIsInteractive);
        c = new GridBagConstraints();
        c.gridx = 2;
        c.gridy = 0;
        c.fill = GridBagConstraints.NONE;
        c.insets = new Insets(2,14,2,0);
        paneSound.add(mButtonClap,c);

        JLabel labelSound = new JLabel(iconSound);
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth = 3;
        c.fill = GridBagConstraints.NONE;
        c.insets = new Insets(6,2,2,2);
        paneSound.add(labelSound,c);

        mBrickControl = new XBrickControl(mIsInteractive);
        c = new GridBagConstraints();
        c.gridx = 1;
        c.gridy = 0;
        c.weightx = 1.0;
        c.weighty = 1.0;
        c.fill = GridBagConstraints.NONE;
        c.insets = stdMargin;
        this.add(mBrickControl,c);

        JPanel paneButtons = new JPanel(new GridLayout(1,3,2,2));
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth = 2;
        c.weightx = 1.0;
        c.insets = new Insets(2,3,2,3);
        c.fill = GridBagConstraints.HORIZONTAL;
        this.add(paneButtons,c);

        mButtonStart = new JButton(XMsg.START);
        mButtonStart.setMnemonic(KeyEvent.VK_S);
        if (mIsInteractive) mButtonStart.addActionListener(this);
        mButtonStart.setEnabled(mIsInteractive);
        paneButtons.add(mButtonStart);

        mButtonPause = new JButton(XMsg.PAUSE);
        mButtonPause.setMnemonic(KeyEvent.VK_P);
        mButtonPause.setEnabled(false);
        if (mIsInteractive) mButtonPause.addActionListener(this);
        mButtonPause.setEnabled(mIsInteractive);
        paneButtons.add(mButtonPause);

        mButtonStop = new JButton(mIsInteractive ? XMsg.CANCEL : XMsg.OK);
        mButtonStop.setMnemonic(KeyEvent.VK_A);
        mButtonStop.addActionListener(this);
        paneButtons.add(mButtonStop);

        // complete initialization depending on current preferences:
        preferencesChanged(XPreferences.getReadInstance());

        XPreferences.addListener(this);
        mSession.addSessionHandler(new SessionHandler());
    }

    public void preferencesChanged(XPreferences.Read newPrefs)
    {
        if (mChangingPreferences) return;

        if (mSliderInputLevel.getValue() != newPrefs.getAudioInputLevel())
        {
            mSliderInputLevel.setValue(newPrefs.getAudioInputLevel());
            mBarInputLevel.setValue(newPrefs.getAudioInputLevel());
        }

        if (mIsInteractive)
        {
            if ( (ESessionState.RUNNING == mSession.getState()) && (mButtonPause.isEnabled() != newPrefs.getSimBreaksEnabled()) )
            {
                mButtonPause.setEnabled(newPrefs.getSimBreaksEnabled());
            }
    
            boolean enableClap = XPreferences.AUDIO_MIXER_OFF.equals(newPrefs.getAudioMixerIn());
            if (mButtonClap.isEnabled() != enableClap)
            {
                mButtonClap.setEnabled(enableClap);
            }
        }
    }

    public void actionPerformed(ActionEvent e)
    {
        if (e.getSource() instanceof JButton)
        {
            JButton button = (JButton) e.getSource();
            if (button == mButtonStart)
            {
                mButtonStart.setEnabled(false);
                if (mSession.getState() == ESessionState.INITIALIZED)
                {
                	mSession.start();
                }
                else if (mSession.getState() == ESessionState.PAUSING)
                {
                	mSession.resume();
                }
                if (XPreferences.getReadInstance().getSimBreaksEnabled()) mButtonPause.setEnabled(true);
            }
            else if (button == mButtonPause)
            {
                if (mSession.getState() == ESessionState.COMPLETED)
                {
                    mButtonPause.setEnabled(false);
                    mButtonPause.setText(XMsg.PAUSE);
                    mButtonStop.setText(XMsg.CANCEL);
                    // See comments in SessionResetHandler:
                	//mSession.init(); does not work properly =>
                    (new SessionResetHandler()).execute();
                }
                else
                {
                    mButtonPause.setEnabled(false);
                    mSession.pause();
                    mButtonStart.setEnabled(true);
                }
            }
            else if (button == mButtonStop)
            {
                XSimulation.terminateSimulationProcess();
            }
        }
    }

    public void stateChanged(ChangeEvent e)
    {
        XPreferences.Write pw = XPreferences.getWriteInstance();
        pw.setAudioInputLevel(mSliderInputLevel.getValue());
        mBarInputLevel.setValue(pw.getAudioInputLevel());
        mChangingPreferences = true;
        pw.store();
        mChangingPreferences = false;
    }

    void updateGraphics()
    {
        SwingUtilities.invokeLater(mUpdateGraphicsHandler);
    }
}
