/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.util.XTimedThread;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Beschreiben Sie hier die Klasse XSimulationView.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
class XSimulationView extends JFrame
{
    class SessionHandler extends BSessionHandler
    {
        @Override
        public String getKey()
        {
            return "XSimulationView.Session#" + sSession.getID();
        }
    
        @Override
        public void terminateSession()
        {
            //mSettingsPanel.storePreferences();
            XPreferences.Write pw = XPreferences.getWriteInstance();
            pw.setViewPosX(mViewPosX);
            pw.setViewPosY(mViewPosY);
            pw.setViewSizeX(mViewSizeX);
            pw.setViewSizeY(mViewSizeY);
            pw.setViewState(getExtendedState() & Frame.MAXIMIZED_BOTH);
            pw.store();
            XPreferences.flush();
        }
    }


    class SimulationViewTrackAndUpdateRunnable implements XTimedThread.IRunnable
    {
        int mCount = 0;
        @Override
        public void init()
        {
        }

        @Override
        public void runTimed()
        {
            mSceneView.trackGraphics(); // XSceneView will decide in trackGraphics() when to call updateGraphics() (resulting in repaints).
                                        // XSceneView.updateGraphics() will also trigger an update of the robot's graphic representation.

            if (++mCount == VIEW_UPDATE_TIME_FACTOR)
            {
                mCount = 0;
                mOverviewPanel.updateGraphics();
                mControlPanel.updateGraphics();
            }
        }
    
        @Override
        public void terminate()
        {
        }

        @Override
        public void shutdown()
        {
            //DEBUG AConsole.close();

            // Fenster schließen?
        }
    }


    class ComponentBoundsHandler extends ComponentAdapter
    {
        @Override
        public void componentMoved(ComponentEvent e)
        {
            if (getExtendedState() == Frame.NORMAL)
            {
                Point location = getLocation();
                mViewPosX = location.x;
                mViewPosY = location.y;
            }
        }

        @Override
        public void componentResized(ComponentEvent e)
        {
            if (getExtendedState() == Frame.NORMAL)
            {
                Dimension dimension = getSize();
                mViewSizeX = dimension.width;
                mViewSizeY = dimension.height;
            }
        }
    }

    class WindowClosingHandler extends WindowAdapter
    {
        @Override
        public void windowClosing(WindowEvent e)
        {
            XSimulation.terminateSimulationProcess();
        }
    }


    // Konstanten / Klassenvariablen
    static final XSession sSession = XSimulationModel.getRobotSession(0);
    static final int VIEW_TRACKING_DELAY = 5;
    static final int VIEW_UPDATE_TIME_FACTOR = 8; // VIEW_UPDATE_TIME = VIEW_UPDATE_TIME_FACTOR * VIEW_TRACKING_DELAY

    // Instanzvariablen - ersetzen Sie das folgende Beispiel mit Ihren Variablen
    private final XSceneView mSceneView;
    private final XOverviewPanel mOverviewPanel;
    private final JTabbedPane mTappedPane;
    private final XSimulationControlPanel mControlPanel;
    private final XSettingsPanel mSettingsPanel;
    private final XInfoPanel mInfoPanel;
    private volatile XTimedThread mViewUpdateThread = null;
    int mViewPosX;
    int mViewPosY;
    int mViewSizeX;
    int mViewSizeY;

    /**
     * Konstruktor für Objekte der Klasse XSimulationView
     */
    public XSimulationView(String title, boolean interactive)
    {
        super(title);

        JPanel content = new JPanel(new GridBagLayout());
        JScrollPane scrollPane = new JScrollPane(content);
        this.add(scrollPane);

        GridBagConstraints c;
        Insets stdMargin = new Insets(2,2,2,2);

        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.gridheight = 2;
        c.weightx = 1.0;
        c.weighty = 1.0;
        c.fill = GridBagConstraints.BOTH;
        c.insets = stdMargin;
        mSceneView = new XSceneView(interactive);
        content.add(mSceneView,c);

        c = new GridBagConstraints();
        c.gridx = 1;
        c.gridy = 0;
        c.weighty = 1.0;
        c.fill = GridBagConstraints.VERTICAL;
        //c.anchor = GridBagConstraints.CENTER;
        c.insets = stdMargin;
        mOverviewPanel = new XOverviewPanel(this,mSceneView);
        content.add(mOverviewPanel,c);

        mTappedPane = new JTabbedPane();
        c = new GridBagConstraints();
        c.gridx = 1;
        c.gridy = 1;
        //c.weighty = 0.0;
        c.fill = GridBagConstraints.NONE;
        //c.anchor = GridBagConstraints.SOUTH;
        c.insets = stdMargin;
        content.add(mTappedPane,c);

        mControlPanel = new XSimulationControlPanel(interactive);
        mTappedPane.add(XMsg.CONTROL_MENU,mControlPanel);

        mSettingsPanel = new XSettingsPanel(interactive);
        mTappedPane.add(XMsg.SETTINGS_MENU,mSettingsPanel);

        mInfoPanel = new XInfoPanel(mTappedPane);
        mTappedPane.add(XMsg.INFO_MENU,mInfoPanel);

        XPreferences.Read p = XPreferences.getReadInstance();
        setPreferredSize(new Dimension(p.getViewSizeX(),p.getViewSizeY()));
        setMinimumSize(new Dimension(600,400));
        mViewPosX = p.getViewPosX();
        mViewPosY = p.getViewPosY();
        setLocation(mViewPosX,mViewPosY);
        setExtendedState(p.getViewState());
        pack();


        //mSceneView.init();
        mOverviewPanel.init();
        mSettingsPanel.init();
        mInfoPanel.init(); // does not automatically start logging (see this.setVisible())

        sSession.addSessionHandler(new SessionHandler());
        this.addComponentListener(new ComponentBoundsHandler());
        this.addWindowListener(new WindowClosingHandler());
        this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);  // see WindowClosingHandler

        // Scrolling will be necessary only in rare cases (very small screens)
        // and in any case will not happen very often.
        // => performance is no issue
        // => use simple scroll mode to avoid painting problems when resizing
        //    a window in scroll mode
        scrollPane.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);

        // No double buffering necessary for main component (XSceneView).
        // If double buffering of root pane turns out to be helpful anyway, use this:
        //this.getRootPane().setDoubleBuffered(true);

        ClassLoader cl = this.getClass().getClassLoader();
        Image iconImage = (new ImageIcon(cl.getResource("resources/images/jabotics-icon.gif"))).getImage();
        this.setIconImage(iconImage);
    }


    @Override
    public void setVisible(boolean visible)
    {
        super.setVisible(visible);

        if (null == mViewUpdateThread)
        {
            mViewUpdateThread = new XTimedThread("Update Simulation View Thread",new SimulationViewTrackAndUpdateRunnable());
            mViewUpdateThread.setDaemon(true);
            mViewUpdateThread.setPriority(Thread.NORM_PRIORITY + 1);
            mViewUpdateThread.setTimeouts(VIEW_TRACKING_DELAY,VIEW_TRACKING_DELAY,VIEW_TRACKING_DELAY);
            mViewUpdateThread.start();

            // Postponed start of logging in info panel (see comments in XInfoPanel):
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    mInfoPanel.startLogging();
                }
            });
        }
    }

    /*void startUpdate()
    {
        if (null == mViewUpdateThread)
        {
            mViewUpdateThread = new XTimedThread("Update Simulation View Thread",new SimulationViewTrackAndUpdateRunnable());
            mViewUpdateThread.setDaemon(true);
            mViewUpdateThread.setPriority(Thread.NORM_PRIORITY + 1);
            mViewUpdateThread.setTimeouts(VIEW_TRACKING_DELAY,VIEW_TRACKING_DELAY,0);
            mViewUpdateThread.start();
        }
    }
    
    void stopUpdate()
    {
        if (null != mViewUpdateThread)  mViewUpdateThread.terminate();
    }


    static public void test()
    {
        //Locale locale = new Locale("en","US");
        Locale locale = new Locale("de","DE");
        XMsg.init(locale);
        XSimulationView view = new XSimulationView("Simulation test", true);
        view.setVisible(true);
        //XTimer.msDelay(5000);
        //System.err.println("Fehler!");
    }*/
}
