/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.defs.*;

/**
 * XSpeaker
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class XSpeaker extends BModelElement
{
    // class variables:
    static private final int MAX_VOLUME = 100;
    static private final float MAX_VOLUME_F = (float)MAX_VOLUME;
    static private final int C2 = 523;
    //static private final int[] PIANO = new int[]{4, 25, 500, 7000, 5};
    //static private final int[] FLUTE = new int[]{10, 25, 2000, 1000, 25};
    //static private final int[] XYLOPHONE = new int[]{1, 8, 3000, 5000, 5};

    // instance variables:
    private volatile float mMasterVolumeRel = 0.5f; // relative value between 0 and 1

    

    // For testing only (see below)! 
    private XSpeaker()
    {
    	super(null);
    }

    XSpeaker(XRobot robot)
    {
    	super(robot.getSession());
    }

    @Override
    public void manageVisitor(BModelVisitor visitor)
    {
        visitor.visitSpeaker(this);
    }

    public int getVolume()
    {
    return Math.round(mMasterVolumeRel * MAX_VOLUME_F);
    }

    public void setVolume(int volume)
    {
        float rel_volume = (float)volume / MAX_VOLUME_F;
        mMasterVolumeRel = (rel_volume < 0f) ? 0f : ((rel_volume > 1f) ? 1f : rel_volume);
    }

    public int beep()
    {
    	XAudioSystem.playTone(XAudioSystem.getTone(600, 200, mMasterVolumeRel));
    	return 600;
    }

    public int twoBeeps()
    {
        // Compare implementation of lejos.nxt.Sound.twoBeeps()
        // and lejos.nxt.Sound.systemSound(boolean aQueued, int aCode).
    	XAudioSystem.ToneDef[] toneDefs = new XAudioSystem.ToneDef[3]; 
    	toneDefs[0] = XAudioSystem.getTone(600, 150, mMasterVolumeRel);
    	toneDefs[1] = XAudioSystem.getTone(600, 50, 0);
        toneDefs[2] = XAudioSystem.getTone(600, 150, mMasterVolumeRel);
        XAudioSystem.playTones(toneDefs);
        return 350;
    }

    public int beepSequence()
    {
        // Compare implementation of lejos.nxt.Sound.beepSequence()
        // and lejos.nxt.Sound.systemSound(boolean aQueued, int aCode).
    	XAudioSystem.ToneDef[] toneDefs = new XAudioSystem.ToneDef[4]; 
        for (int i = 0; i < 4; ++i)
        {
        	toneDefs[i] = XAudioSystem.getTone(C2 * (7-i) / 4, 100, mMasterVolumeRel);
        }
        XAudioSystem.playTones(toneDefs);
        return 400;
    }

    public int beepSequenceUp()
    {
        // Compare implementation of lejos.nxt.Sound.beepSequenceUp()
        // and lejos.nxt.Sound.systemSound(boolean aQueued, int aCode).
    	XAudioSystem.ToneDef[] toneDefs = new XAudioSystem.ToneDef[4]; 
        for (int i = 0; i < 4; ++i)
        {
        	toneDefs[i] = XAudioSystem.getTone(C2 * (4+i) / 4, 100, mMasterVolumeRel);
        }
        XAudioSystem.playTones(toneDefs);
        return 400;
    }

    public int buzz()
    {
    	XAudioSystem.playTone(XAudioSystem.getModulatedTone(450, 50, 1f, 500, mMasterVolumeRel));
    	return 500;
    }

    public void tone(int frequency, int period)
    {
    	XAudioSystem.playTone(XAudioSystem.getTone(frequency, period, mMasterVolumeRel));
    }

    public void tone(int frequency, int period, int volume)
    {
        if (volume < 0)
        {
            volume = 0;
        }
        else if (volume > MAX_VOLUME)
        {
            volume = MAX_VOLUME;
        }

        XAudioSystem.playTone(XAudioSystem.getTone(frequency, period, (mMasterVolumeRel*volume)/MAX_VOLUME_F));
    }

    public void note(ESound instrument, int frequency, int period)
    {
        switch (instrument)
        {
            case PIANO:
            {
            	XAudioSystem.playTone(XAudioSystem.getModulatedTone(frequency, 2*frequency, 0.04f, period, mMasterVolumeRel, 5, 2000, 0f));
                break;
            }
            case FLUTE:
            {
            	XAudioSystem.playTone(XAudioSystem.getModulatedTone(frequency, frequency / 70 , 0.1f, period, mMasterVolumeRel, 10, 200, 0.5f));
                break;
            }
            default:
            {
                // XYLOPHONE
            	XAudioSystem.playTone(XAudioSystem.getTone(frequency, period, mMasterVolumeRel, 2, 300, 0f));
                break;
            }
        }
    }


    // Needs to be called from XSimulationModel.main() 
    /*static public void testTone()
    {
        XSpeaker speaker = new XSpeaker();

        XSession.msDelay(speaker.beep());
        XSession.msDelay(1000);
        XSession.msDelay(speaker.twoBeeps());
        XSession.msDelay(1000);
        XSession.msDelay(speaker.buzz());
        XSession.msDelay(1000);
        XSession.msDelay(speaker.beepSequence());
        XSession.msDelay(1000);
        XSession.msDelay(speaker.beepSequenceUp());
        XSession.msDelay(1000);
        speaker.tone(440,1000);
        XSession.msDelay(2000);
        speaker.note(ESound.PIANO,440,1000);
        XSession.msDelay(2000);
        speaker.note(ESound.FLUTE,440,1000);
        XSession.msDelay(2000);
        speaker.note(ESound.XYLOPHONE,440,1000);
        XSession.msDelay(2000);
        XAudioSystem.playRobotSignal();
    }*/
}
