/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.defs.*;

import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.geom.Rectangle2D;

/**
 * Beschreiben Sie hier die Klasse XWheel.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class XWheel extends BModelElement
{
    // Instanzvariablen
    private EMotorPort mPort;
    private XMotor.MotorSet mMotorSet;
    private final XGraphicsShape mGraphicsWheel;

    XWheel(XRobot robot, EMotorPort port, XMotor.MotorSet motor_set)
    {
        super(robot.getSession());
        mPort = port;
        mMotorSet = motor_set;

        // build graphics representation:
        mGraphicsWheel = new XGraphicsShape(new Rectangle2D.Float(-2.4f,-1.1f,4.8f,2.2f));
        mGraphicsWheel.setPaint(Color.darkGray);
        mGraphicsWheel.setDraw(false);
        mGraphicsWheel.setFill(true);
        mGraphicsRep.add(mGraphicsWheel);
    }

    void setPort(EMotorPort port)
    {
        mPort = port;
    }

    @Override
    public void updateGraphics()
    {
        XMotor.RotationData data = mMotorSet.getGraphicsData();
        float speed = (float)data.getRotSpeed(mPort);
        float angle = (float)restrictedAngle(data.getRotAngle(mPort));
        float deltaBrightness = 20f / (100f + speed * speed); // means: 0.2f * (100f / (100f + speed * speed));
        GradientPaint gradient = new GradientPaint(4.8f*angle/D.PI_F,0f,Color.getHSBColor(0f,0f,0.3f-deltaBrightness),4.8f*angle/D.PI_F+2.4f,0f,Color.getHSBColor(0f,0f,0.3f+deltaBrightness),true);
        mGraphicsWheel.setPaint(gradient);
    }

    @Override
    public void manageVisitor(BModelVisitor visitor)
    {
        visitor.visitWheel(this);
    }

    private double restrictedAngle(double angle)
    {
        return ((angle >= D.TWO_PI) || (angle < 0)) ? (angle - D.TWO_PI*Math.floor(angle/D.TWO_PI)) : angle;
    }
}
