/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.test.core;

import org.jabotics.robot.util.*;


/**
 * Beschreiben Sie hier die Klasse TestUtil.
 * 
 * @author (Ihr Name) 
 * @version (eine Versionsnummer oder ein Datum)
 */
public class TestUtil
{
	static final long START_ID = 325;

	static boolean sUnitTest = true;

	static class XData implements TICloneable<XData>
    {
        long mID = START_ID;

        public XData getClone()
        {
            XData clone = new XData();
            clone.copyDataFrom(this);
            return clone;
        }

        public void copyDataFrom(XData template)
        {
            mID = template.mID + 1;            
        }
    }

	static class TestGate
	{
		XData mData;
		TGate<XData> mGate;
		volatile XData mDataRead = null;
		volatile XData mDataWrite = null;
		volatile XData mDataWork = null;
		volatile long mReadCnt = 0;
		volatile long mReadCntTotal = 0;
		volatile long mReadID = 0;
		volatile long mWriteCnt = 0;
		volatile int mErrorCntRead = 0;
		volatile int mErrorCntWrite = 0;
		volatile boolean mRun = false;


		public TestGate()
		{
			mData = new XData();
			mGate = new TGate<XData>(mData);
		}


	    public void test()
	    {
	        mData = mGate.write(mData);
	        if (sUnitTest)
	            assert mData.mID == (START_ID+1) : "ID of data object does not meet expection (expected: " + (START_ID+1) + "; is " + mData.mID + ").";
	        else
	        	System.out.println(mData.mID);

	        mData = mGate.read();
	        if (sUnitTest)
	            assert mData.mID == (START_ID+1) : "ID of data object does not meet expection (expected: " + (START_ID+1) + "; is " + mData.mID + ").";
	        else
	        	System.out.println(mData.mID);

	        mData = mGate.write(mData);
	        if (sUnitTest)
	            assert mData.mID == (START_ID+2) : "ID of data object does not meet expection (expected: " + (START_ID+2) + "; is " + mData.mID + ").";
	        else
	        	System.out.println(mData.mID);

	        mData = mGate.read();
	        if (sUnitTest)
	            assert mData.mID == (START_ID+2) : "ID of data object does not meet expection (expected: " + (START_ID+2) + "; is " + mData.mID + ").";
	        else
	        	System.out.println(mData.mID);

	
	        mDataWork = mGate.getWriteObject();
	        mDataWork.mID = 0;
	        mDataWork = null;
	        mDataWrite = mGate.commitWriteObject();
	        mDataRead = mGate.read();
	  
	        mRun = true;
	        Thread writer = new Thread(new Writer());
	        Thread reader = new Thread(new Reader());
	        writer.start();
	        reader.start();
	        pause(20000);
	        mRun = false;
	        pause(500);

	        if (sUnitTest)
	        {
	            assert mErrorCntRead == 0 : "Access violation or other error detected in Reader thread (error count = " + mErrorCntRead + " for about " + mReadCnt + " access cycles.";
	            assert mErrorCntWrite == 0 : "Access violation or other error detected in Writer thread (error count = " + mErrorCntWrite + " for about " + mWriteCnt + " access cycles.";
	            assert mWriteCnt == mDataWrite.mID : "Write count and ID of write data element should be identical (write count = " + mWriteCnt + " and data id = " + mDataWrite.mID + ").";
	            assert mWriteCnt > 60000000 : "Write count does not meet expectation for reference machine (write count = " + mWriteCnt + ").";
	        }
	        else
	        {
		        System.out.println("Leseversuche:    " + mReadCntTotal);
		        System.out.println("Lesezugriffe:    " + mReadCnt);
		        System.out.println("Schreibzugriffe: " + mWriteCnt);
		        System.out.println("Fehler (Reader): " + mErrorCntRead);
		        System.out.println("Fehler (Writer): " + mErrorCntWrite);
		        System.out.println("Letzte Daten-ID: " + mDataWrite.mID);
		    }
	    }

	    class Writer implements Runnable
	    {
	    	public void run()
	    	{
	    		while (mRun)
	    		{
					++mWriteCnt;
	    			mDataWork = mGate.getWriteObject();
					if ((mDataWork == mDataRead) || (mWriteCnt != mDataWork.mID))
					{
						++mErrorCntWrite;
					}
	    			mDataWork = null; // necessary for a proper detection of access violations
	    			mDataWrite = mGate.commitWriteObject();
	    		}
	    	}
	    }
	
	    class Reader implements Runnable
	    {
	    	public void run()
	    	{
	    		while (mRun)
	    		{
	    			if (mGate.isModified())
	    			{
	    				do
		    			{
		    				++mReadCntTotal;
		    				++mReadCnt;
		    				mDataRead = null; // necessary for a proper detection of access violations
		    				mDataRead = mGate.read();
		    				if ((mDataRead == mDataWork) || (mReadID >= mDataRead.mID))
		    				{
		    					++mErrorCntRead;
		    				}
		    				mReadID = mDataRead.mID;
		    			}
		    			while (mGate.isModified());
	    			}
	    			else
	    			{
	    				++mReadCntTotal;
	    			}
	    		}
	    	}
	    }
	}


	static class TestPipe
	{
		XData mData;
		TPipe<XData> mPipe;
		volatile XData mDataRead = null;
		volatile XData mDataWrite = null;
		volatile XData mDataWork = null;
		volatile long mReadCnt = 0;
		volatile long mWriteCnt = 0;
		volatile int mErrorCntRead = 0;
		volatile int mErrorCntWrite = 0;
		volatile boolean mRun = false;


		public TestPipe()
		{
			mData = new XData();
			mPipe = new TPipe<XData>(mData);
		}


	    public void test()
	    {
	        mData = mPipe.write(mData);
	        if (sUnitTest)
	            assert mData.mID == (START_ID+1) : "ID of data object does not meet expection (expected: " + (START_ID+1) + "; is " + mData.mID + ").";
	        else
	        	System.out.println(mData.mID);

	        mData = mPipe.write(mData);
	        if (sUnitTest)
	            assert mData.mID == (START_ID+2) : "ID of data object does not meet expection (expected: " + (START_ID+2) + "; is " + mData.mID + ").";
	        else
	        	System.out.println(mData.mID);

	        mData = mPipe.read();
	        if (sUnitTest)
	            assert mData.mID == (START_ID+1) : "ID of data object does not meet expection (expected: " + (START_ID+1) + "; is " + mData.mID + ").";
	        else
	        	System.out.println(mData.mID);

	        mData = mPipe.read();
	        if (sUnitTest)
	            assert mData.mID == (START_ID+2) : "ID of data object does not meet expection (expected: " + (START_ID+2) + "; is " + mData.mID + ").";
	        else
	        	System.out.println(mData.mID);

	
	        mDataWork = mPipe.getWriteObject();
	        mDataWork.mID = 0;
	        mDataWork = null;
	        mDataWrite = mPipe.commitWriteObject();
	        mDataRead = mPipe.read();
	  
	        mRun = true;
	        Thread writer = new Thread(new Writer());
	        Thread reader = new Thread(new Reader());
	        writer.start();
	        reader.start();
	        pause(20000);
	        mRun = false;
	        pause(500);

	        if (sUnitTest)
	        {
	            assert mErrorCntRead == 0 : "Access violation or other error detected in Reader thread (error count = " + mErrorCntRead + " for about " + mReadCnt + " access cycles.";
	            assert mErrorCntWrite == 0 : "Access violation or other error detected in Writer thread (error count = " + mErrorCntWrite + " for about " + mWriteCnt + " access cycles.";
	            assert mWriteCnt == mDataWrite.mID : "Write count and ID of write data element should be identical (write count = " + mWriteCnt + " and data id = " + mDataWrite.mID + ").";
	            assert mWriteCnt > 80000000 : "Write count does not meet expectation for reference machine (write count = " + mWriteCnt + ").";
	            assert mPipe.getCapacity()< 500 : "Pipe capacity exceeds expectation for reference machine (capacity = " + mPipe.getCapacity() + ").";
	        }
	        else
	        {
		        System.out.println("Lesezugriffe:    " + mReadCnt);
		        System.out.println("Schreibzugriffe: " + mWriteCnt);
		        System.out.println("Fehler (Reader): " + mErrorCntRead);
		        System.out.println("Fehler (Writer): " + mErrorCntWrite);
		        System.out.println("Letzte Daten-ID: " + mDataWrite.mID);
		        System.out.println("Pipe-Kapazität:  " + mPipe.getCapacity());
		    }
	    }

	    class Writer implements Runnable
	    {
	    	public void run()
	    	{
	    		while (mRun)
	    		{
					++mWriteCnt;
	    			mDataWork = mPipe.getWriteObject();
					if ((mDataWork == mDataRead) || (mWriteCnt != mDataWork.mID))
					{
						++mErrorCntWrite;
					}
	    			mDataWork = null; // necessary for a proper detection of access violations
	    			mDataWrite = mPipe.commitWriteObject();
	    			int size = mPipe.getSize();
	    			if (size > 100)
	    			{
	    				// dummy calculation to slow down writing procedure
	    				double x = 0, y = 0;
	    				for (int i = 0; i < size; ++i)
	    				{
	    					y = x;
	    					x = y + Math.sin(i);
	    				}
	    				//Thread.yield();
	   				}
	    		}
	    	}
	    }
	
	    class Reader implements Runnable
	    {
	    	public void run()
	    	{
	    		while (mRun)
	    		{
	    			while (mPipe.hasNext())
	    			{
	    				mDataRead = null; // necessary for a proper detection of access violations
	    				mDataRead = mPipe.read();
	    				if ((mDataRead == mDataWork) || (++mReadCnt != mDataRead.mID))
	    				{
	    					++mErrorCntRead;
	    				}
	    			}
	    			Thread.yield();
	    		}
	    	}
	    }
	}


	static public void main(String[] args)
    {
    	sUnitTest = false;

    	TestGate testGate = new TestGate();
    	testGate.test();

    	TestPipe testPipe = new TestPipe();
    	testPipe.test();
    }


    private static void pause(long period_ms)
    {
        try
        {
            Thread.sleep(period_ms);
        }
        catch (InterruptedException e) {}
    }
}
