/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.test.ipc;

import static org.junit.Assert.*;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.jabotics.pc.util.XSharedData;

public class UTestSharedMemory
{
    static final int TEST_PERIOD = 10000;
    static final int READS = 0;
    static final int ERRORS = 1;
    static final int REPEATS = 2;
    static final boolean WINDOWS = ('\\' == File.separatorChar);


    static class SharedDataProvider
    {
        XSharedData           mSD;
        XSharedData.XInt      mInteger1;
        XSharedData.XInt      mCntProvider1;
        XSharedData.XInt      mInteger2;
        XSharedData.XInt      mCntProvider2;

        SharedDataProvider(String name, boolean master)
        {
            mSD = new XSharedData(name, master);
            mInteger1 = new XSharedData.XInt(mSD);
            mCntProvider1 = new XSharedData.XInt(mSD);
            mInteger2 = new XSharedData.XInt(mSD);
            mCntProvider2 = new XSharedData.XInt(mSD);
            mSD.create();
        }
    }

    static class SharedRandomInts
    {
        XSharedData           mSD;
        XSharedData.XIntArray mRandomInts1;
        XSharedData.XIntArray mRandomInts2;

        SharedRandomInts(String name, boolean master)
        {
            mSD = new XSharedData(name, master);
            mRandomInts1 = new XSharedData.XIntArray(mSD,256);
            mRandomInts2 = new XSharedData.XIntArray(mSD,256);
            mSD.create();
        }
    }

    static class SharedDataConsumer
    {
        XSharedData           mSD;
        XSharedData.XIntArray mCounts;

        SharedDataConsumer(String name, boolean master)
        {
            mSD = new XSharedData(name, master);
            mCounts = new XSharedData.XIntArray(mSD,3);
            mSD.create();
        }
    }

    public static void main(String[] args)
    {
        if (args.length == 2)
        {
            String name = args[0];
            String role = args[1];
            if ((null == name) || (null == role)) System.exit(1);

            switch (role)
            {
                 case "Provider":
                {
                    final SharedDataProvider data0 = new SharedDataProvider(name,false);
                    final SharedRandomInts  ridata = new SharedRandomInts(name+"-RandomInts",false);
                    ridata.mSD.read();
                    
                    (new Thread() {public void run() {
                        int count = 0;
                        int i = 0;
                        long end = System.currentTimeMillis() + TEST_PERIOD;
                        while (System.currentTimeMillis() < end)
                        {
                            data0.mInteger2.set(ridata.mRandomInts2.get(i++));
                            data0.mSD.write();
                            if (ridata.mRandomInts2.size() == i)  i = 0;
                            count++;
                        }
                        data0.mCntProvider2.set(count);
                        data0.mSD.write();}}).start();
                    

                    int count = 0;
                    int i = 0;
                    long end = System.currentTimeMillis() + TEST_PERIOD;
                    while (System.currentTimeMillis() < end)
                    {
                        data0.mInteger1.set(ridata.mRandomInts1.get(i++));
                        data0.mSD.write();
                        if (ridata.mRandomInts1.size() == i)  i = 0;
                        count++;
                    }
                    data0.mCntProvider1.set(count);
                    data0.mSD.write();

                    break;
                }
    
                case "Consumer1":
                {
                    SharedDataProvider data0 = new SharedDataProvider(name,false);
                    SharedDataConsumer data1 = new SharedDataConsumer(name+"-Consumer1",false);
                    SharedRandomInts  ridata = new SharedRandomInts(name+"-RandomInts",false);
                    final int size = ridata.mRandomInts1.size();
                    HashSet<Integer> set = new HashSet<>(2*size);
                    ridata.mSD.read();
                    for (int i = 0; i < size; set.add(ridata.mRandomInts1.get(i++))); 
    
                    int count = 0;
                    int errors = 0;
                    int lastValue = 0;
                    long end = System.currentTimeMillis() + TEST_PERIOD;
                    while (System.currentTimeMillis() < end)
                    {
                        data0.mSD.read();
                        int value = data0.mInteger1.get();
                        if (!set.contains(value)) errors++;
                        if (value != lastValue) {count++; lastValue = value;}
                    }
                    data1.mCounts.set(READS,count);
                    data1.mCounts.set(ERRORS,errors);
                    data1.mCounts.set(REPEATS,(int)data0.mSD.getInconsistentReadCount());
                    data1.mSD.write();

                    break;
                }
    
                /*case "Provider2":
                {
                    SharedData data = new SharedData(name,false);
                    int size = data.mRandomInts2.size();
                    int[] randomInts = new int[size];
                    for (int i = size; i != 0; randomInts[--i] = (int) (Math.random() * Integer.MAX_VALUE));
                    data.mRandomInts2.set(randomInts);

                    pause(1000);
    
                    int count = 0;
                    int i = 0;
                    long end = System.currentTimeMillis() + TEST_PERIOD;
                    while (System.currentTimeMillis() < end)
                    {
                        data.mInteger2.set(randomInts[i++]);
                        if (data.mRandomInts2.size() == i)  i = 0;
                        count++;
                    }
                    data.mCntProvider2.set(count);
                    
                    break;
                }*/
    
                default: // case "Consumer2"
                {
                    SharedDataProvider data0 = new SharedDataProvider(name,false);
                    SharedDataConsumer data2 = new SharedDataConsumer(name+"-Consumer2",false);
                    SharedRandomInts ridata = new SharedRandomInts(name+"-RandomInts",false);
                    final int size = ridata.mRandomInts2.size();
                    HashSet<Integer> set = new HashSet<>(2*size);
                    ridata.mSD.read();
                    for (int i = 0; i < size; set.add(ridata.mRandomInts2.get(i++))); 
    
                    int count = 0;
                    int errors = 0;
                    int lastValue = 0;
                    long end = System.currentTimeMillis() + TEST_PERIOD;
                    while (System.currentTimeMillis() < end)
                    {
                        data0.mSD.read();
                        int value = data0.mInteger2.get();
                        if (!set.contains(value)) errors++;
                        if (value != lastValue) {count++; lastValue = value;}
                    }
                    data2.mCounts.set(READS,count);
                    data2.mCounts.set(ERRORS,errors);
                    data2.mCounts.set(REPEATS,(int)data0.mSD.getInconsistentReadCount());
                    data2.mSD.write();

                    break;
                }
            }
        }
        else
        {
            UTestSharedMemory testSharedMemory = new UTestSharedMemory(); 
            testSharedMemory.test();
        }
    }

    @BeforeClass
    public static void setUpBeforeClass() throws Exception
    {
    }

    @AfterClass
    public static void tearDownAfterClass() throws Exception
    {
    }

    @Before
    public void setUp() throws Exception
    {
    }

    @After
    public void tearDown() throws Exception
    {
    }

    @Test(timeout=16000)
    public void test()
    {
        String memoryName = "SharedMemoryTest";

        // working directory (java property "user.dir") is project directory in Eclipse
        String classpath = System.getProperty("jabotics.classpath");
        //String classpath = System.getProperty("user.dir") + File.separatorChar + "bin";
        String java = System.getProperty("java.home") + (WINDOWS ? "\\bin\\java.exe" : "/bin/java");

        SharedDataProvider data0 = new SharedDataProvider(memoryName,true);
        SharedDataConsumer data1 = new SharedDataConsumer(memoryName+"-Consumer1",true);
        SharedDataConsumer data2 = new SharedDataConsumer(memoryName+"-Consumer2",true);
        SharedRandomInts  ridata = new SharedRandomInts(memoryName+"-RandomInts",true);

        int size = ridata.mRandomInts1.size();
        for (int i = 0; i < size; ridata.mRandomInts1.set(i++, (int) (Math.random() * Integer.MAX_VALUE)));
        size = ridata.mRandomInts2.size();
        for (int i = 0; i < size; ridata.mRandomInts2.set(i++, (int) (Math.random() * Integer.MAX_VALUE)));
        ridata.mSD.write();
        
        try
        {
            ArrayList<String> commandList = new ArrayList<String>();
            ProcessBuilder pb = new ProcessBuilder(commandList);;
            commandList.add(java);
            commandList.add("-classpath");
            commandList.add(classpath);
            commandList.add("org.jabotics.test.ipc.UTestSharedMemory");
            commandList.add(memoryName);
            int pos_role = commandList.size();
            commandList.add("Provider");

            Process p1 = pb.start();
            if (!p1.isAlive()) return;

            commandList.set(pos_role,"Consumer1");
            Process p2 = pb.start();
            if (!p2.isAlive()) return;

            /*commandList.set(pos_role,"Provider2");
            Process p3 = pb.start();
            if (!p3.isAlive()) return;*/
        
            commandList.set(pos_role,"Consumer2");
            Process p4 = pb.start();
            if (!p4.isAlive()) return;
            
            while (p1.isAlive() || p2.isAlive() || p4.isAlive()) pause(100);
        }
        catch (IOException e)
        {
            fail("Process could not be started.");
        }

        data0.mSD.read();
        int cntProvider1 = data0.mCntProvider1.get();
        int cntProvider2 = data0.mCntProvider2.get();
        data1.mSD.read();
        int cntConsumer1 = data1.mCounts.get(READS);
        int cntErrors1 = data1.mCounts.get(ERRORS);
        int cntRepeats1 = data1.mCounts.get(REPEATS);
        data2.mSD.read();
        int cntConsumer2 = data2.mCounts.get(READS);
        int cntErrors2 = data2.mCounts.get(ERRORS);
        int cntRepeats2 = data2.mCounts.get(REPEATS);
        assert cntErrors1 == 0: "Read errors have been detected (error count is " + cntErrors1 + ").";
        assert cntProvider1 > 2000000 : "Iteration count of provider process does not meet minimum expectation for reference PC (is " + cntProvider1 + ").";
        assert cntConsumer1 > 2000000 : "Iteration count of consumer process does not meet minimum expectation for reference PC (is " + cntConsumer1 + ").";
        assert cntErrors2 == 0: "Read errors2 have been detected (error count is " + cntErrors2 + ").";
        assert cntProvider2 > 2000000 : "Iteration count of provider2 process does not meet minimum expectation for reference PC (is " + cntProvider2 + ").";
        assert cntConsumer2 > 2000000 : "Iteration count of consumer2 process does not meet minimum expectation for reference PC (is " + cntConsumer2 + ").";
        assert cntRepeats1 < 200000: "Inconsistent reads exceed expactations for reference system (count is " + cntRepeats1 + ").";
        assert cntRepeats2 < 200000: "Inconsistent reads exceed expactations for reference system (count is " + cntRepeats2 + ").";

    }

    private static void pause(long period_ms)
    {
        try
        {
            Thread.sleep(period_ms);
        }
        catch (InterruptedException e)
        {
            fail("Thread has been interrupted.");
        }
    }
}
