/********************************************************************************************
*
* Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
*
* 
* This file is part of the Java software package org.jabotics (in the following: Jabotics).
* 
* Jabotics is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* Jabotics is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
* 
* 
* Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
* 
* Jabotics ist Freie Software: Sie können es unter den Bedingungen
* der GNU General Public License, wie von der Free Software Foundation,
* Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
* veröffentlichten Version, weiterverbreiten und/oder modifizieren.
* 
* Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
* OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
* Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
* Siehe die GNU General Public License für weitere Details.
* 
* Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
* erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
* 
*******************************************************************************************/


package org.jabotics.test.ipc;

import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.Naming;
import java.rmi.RemoteException;

public class XTestRMIServer extends UnicastRemoteObject implements ITestRMIServer
{
    public static final int DEFAULT_RMI_PORT = 1099; 
    public static final int PORT = DEFAULT_RMI_PORT;
    //public static final int PORT = 1541;
    public static final String REGISTRY_NAME = "//localhost:" + PORT; 
    public static final String SERVER_NAME = REGISTRY_NAME + "/TestRMIServer";

    boolean mTerminating = false;

    public XTestRMIServer() throws RemoteException
    {
        super(PORT);    // required to avoid the 'rmic' step
    }

    /*
     * The worker instance is running in the server process
     */
    @Override
    public ITestRMIWorker getWorker()
    {
        ITestRMIWorker worker = null;
        try
        {
            worker = new XTestRMIWorker("Worker");
        }
        catch (Exception e) {}

        return worker;
    }

    @Override
    public void terminate()
    {
        if (mTerminating) return;
        mTerminating = true;
        (new Thread(new Terminater())).start();
    }

    class Terminater implements Runnable
    {
        public void run()
        {
            try
            {
                pause(500);
                UnicastRemoteObject.unexportObject(XTestRMIServer.this, true);
            }
            catch (RemoteException e) {}

            System.exit(0); // will terminate registry also
        }
    }
 
    public static void main(String[] args)
    {
        try
        {
            // Make sure that rmi registry is running
            LocateRegistry.createRegistry(PORT);
        }
        catch (RemoteException e) {}

        try
        {
            Naming.rebind(SERVER_NAME, new XTestRMIServer());
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    private static void pause(long period_ms)
    {
        try
        {
            Thread.sleep(period_ms);
        }
        catch (InterruptedException e) {}
    }
}
