/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 * 
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/

package org.jabotics.test.robot;


import static org.jabotics.pc.util.Assert.*;

/**
 * Beschreibe hier die Klasse TestOutOfHere.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class TestOutOfHere extends BTestRoboter
{
    static final int ABSTAND_SOLL = 22;
    static final int ABSTAND_INNEN = 20;
    static final int ABSTAND_AUSSEN = 25;
    //static final int ABSTAND_KURVE = 40;
    static final int V_SOLL = 10;
    static final int A_SOLL = 40;
    static final int V_SOLL_RUECK = 5;
    static final int A_SOLL_RUECK = 20;
    static final int PERIODE = 100;
    static final int T_INTEGRATION = 400;
    static final float UEBERSTEUERUNGSFAKTOR = 1.5f;
    static final float GEWICHT_NEU = (float)PERIODE / (float)(PERIODE + T_INTEGRATION);
    static final float GEWICHT_ALT = 1f - GEWICHT_NEU;

    long mStartzeitpunkt;
    long mEndzeitpunkt;

    float mPos = 0f;
    float mAbstand = 0f;
    float mStrecke = 0f;
    float mDiff = 0f;

    public static void main(String[] args)
    {
        TestOutOfHere roboter = new TestOutOfHere();
        roboter.run();
    }

    /**
     * Hier können bei Bedarf zusätzlich Initialisierungen einer mit dem new-Operator
     * erzeugten Roboterinstanz vorgenommen werden.
     */
    public TestOutOfHere()
    {
        // In diesem Falle müssen die Instanzvariablen nicht initialisiert werden.
        // Die Initialisierung der Variablen sMin und wMin erfolgt an geeigneter
        // Stelle in der run()-Methode.
    }

    @Override
    public void run()
    {
        mStartzeitpunkt = uhr.holeSystemzeit();

        uhr.warteZeit(2000);

        init();

        do
        {
            if (tastsensor.warteBisGedrueckt(200))
            {
                // Hindernis!!!
                setzeZurueckUndFahreRechtskurve();
            }
            else
            {
                float posAlt = mPos;
                float abstandAlt = mAbstand;
                mPos = pilot.holeTachostand();
                mAbstand = ultraschallsensor.holeMesswert();
                mStrecke = GEWICHT_NEU * (posAlt - mPos) + GEWICHT_ALT * mStrecke; // "gewichtete", zu mDiff passende Strecke (Rückwärtsfahrt)
                mDiff = GEWICHT_NEU * ((float)(mAbstand - abstandAlt)) + GEWICHT_ALT * mDiff;
                if (mStrecke > 1f)
                {
                    float winkel = UEBERSTEUERUNGSFAKTOR * (float)Math.toDegrees(Math.atan(mDiff/mStrecke));

                    if ((mAbstand > ABSTAND_AUSSEN) && (winkel > -1))
                    {
                        korrigiereRichtung((winkel < 5f) ? 5f : winkel);
                    }
                    else if ((mAbstand < ABSTAND_INNEN) && (winkel < 1))
                    {
                        korrigiereRichtung((winkel > -5f) ? -5f : winkel);
                    }
                }
            }
        }
        while (lichtsensor.holeMesswert() < 85f);

        pilot.halteAn(STARTEN);
        lautsprecher.spieleBeepfolgeAufwaerts();

        uhr.warteZeit(2000);

        mEndzeitpunkt = uhr.holeSystemzeit();
    }

    private void init()
    {
        lautsprecher.setzeLautstaerke(30);
        ultraschallsensor.setzeTraegheit(0);
        pilot.setzeSollgeschwindigkeit(V_SOLL);
        pilot.setzeSollbeschleunigung(A_SOLL);
        pilot.fahreGerade(RUECKWAERTS);
        mPos = pilot.holeTachostand();
        mAbstand = ultraschallsensor.holeMesswert();
        mDiff = 0f;
        mStrecke = 0f;
    }

    private void korrigiereRichtung(float winkel)
    {
        if (winkel > 30f) winkel = 30f;
        if (winkel < -30f) winkel = -30f;
        pilot.halteAn(ABWARTEN);
        pilot.dreheUmWinkel(winkel, ABWARTEN);
        pilot.fahreGerade(RUECKWAERTS);
        mStrecke = 0f;
        mDiff = 0f;
        mPos = pilot.holeTachostand();
        mAbstand = ultraschallsensor.holeMesswert();
    }

    private void setzeZurueckUndFahreRechtskurve()
    {
        pilot.halteAn(ABWARTEN);
        pilot.setzeSollgeschwindigkeit(V_SOLL_RUECK);
        pilot.setzeSollbeschleunigung(A_SOLL_RUECK);

        lichtsensor.schalteLichtEin();
        lautsprecher.spieleBeep();
        lichtsensor.schalteLichtAus();
        lautsprecher.spielePause(700);

        pilot.fahreGerade(ABSTAND_SOLL-15,STARTEN);

        lichtsensor.schalteLichtEin();
        lautsprecher.spieleBeep();
        lichtsensor.schalteLichtAus();
        lautsprecher.spielePause(700);
        lichtsensor.schalteLichtEin();
        lautsprecher.spieleBeep();
        lichtsensor.schalteLichtAus();
        lautsprecher.spielePause(700);
        lichtsensor.schalteLichtEin();
        lautsprecher.spieleBeep();
        lichtsensor.schalteLichtAus();
        lautsprecher.spielePause(700);

        pilot.dreheUmWinkel(-90,STARTEN);

        lichtsensor.schalteLichtEin();
        lautsprecher.spieleBeep();
        lichtsensor.schalteLichtAus();
        lautsprecher.spielePause(700);
        lichtsensor.schalteLichtEin();
        lautsprecher.spieleBeep();
        lichtsensor.schalteLichtAus();
        lautsprecher.spielePause(700);
        lichtsensor.schalteLichtEin();
        lautsprecher.spieleBeep();
        lichtsensor.schalteLichtAus();
        lautsprecher.spielePause(700);

        pilot.setzeSollgeschwindigkeit(V_SOLL);
        pilot.setzeSollbeschleunigung(A_SOLL);
        pilot.fahreGerade(RUECKWAERTS);
        mStrecke = 0f;
        mDiff = 0f;
        mPos = pilot.holeTachostand();
        mAbstand = ultraschallsensor.holeMesswert();
    }

    public void trailer()
    {
        uhr.warteZeit(2000);

        boolean invertiert = false;
        for (int i = 0;  i < 8;  ++i)
        {
            invertiert = !invertiert;
            monitor.schreibe("Raus hier!", 3, 2, invertiert);
            monitor.schreibe("Out of here!", 2, 4, invertiert);
            uhr.warteZeit(500);
        }
    }

    @Override
    public void checkAsserts()
    {
        /*System.out.println("TestOutOfHere:");
        System.out.println("Start:   " + mStartzeitpunkt);
        System.out.println("Ende:    " + mEndzeitpunkt);
        System.out.println("Dauer:   " + (mEndzeitpunkt - mStartzeitpunkt));
        System.out.println("Strecke: " + pilot.holeTachostand());
        System.out.println("Winkel:  " + pilot.holeAusrichtungswinkel());*/

        assertEquals("Total distance does not meet expection.", -2380f, pilot.holeTachostand(), 50f);
        assertEquals("Total robot rotation does not meet expection.", 23f, pilot.holeAusrichtungswinkel(), 20f);
        assertEquals("Execution time does not meet expection.", 327000, (mEndzeitpunkt - mStartzeitpunkt), 10000);
    }
}
